
#include <stdio.h>
#include <stdlib.h>
#include <curses.h>
#include <unistd.h>

#include <common.h>
#include <client.h>

#include "disp.h"

extern BBS_Config *config;
extern BBS_Node *this_node;

/* lists all areas available to a user */
void list_fileareas(void) {

     int len, i, x, y;
     FILE_Area *list;
     int paused = 3;

     if((list = file_arealist(&len)) == NULL) {
	  bbserr("Server error, or no areas configured.\n");
	  return;
     }

     clear();

     printw("%-20s %-59s\n", "Area Name", "Description");
     printw("-------------------- -------------------------------------------------------\n");

     i = 0;

     while(i < len) {
	  if(list[i].read > this_node->current_user->access) {
	       i++;
	       continue;
	  }

	  printw("%-20s %-58s\n", list[i].name, list[i].desc);
	  i++;
	  paused++;

	  /* bottom of screen? */
	  getyx(stdscr, y, x);
	  if(y > LINES-3) {
	       key_pause();
	       clear();
	  }
	  
     }

     key_pause();

     free(list);

}

void list_files(void) {
     
     FILE_Info *list;
     int len;
     int i, x, y;

     if((list = file_list(this_node->filearea, &len)) == NULL) {
	  bbserr("There are no files in this area.\n");
	  return;
     }

     clear();
     i = 0;

     while(i < len) {
	  char *progress;
	  char *laststop;
	  char buffer[COLS];

	  /* check if we're too low at the start */
	  getyx(stdscr, y, x);
	  if(y > LINES-3) {
	       key_pause();
	       clear();
	  }
	       

	  printw("\n%8d %s\n", list[i].size, list[i].name);

	  /* get the full data on this file */
	  if(file_info(&(list[i])) == -1) {
	       i++;
	       continue;
	  }

	  /* can we fit this on one line? */
	  if(list[i].desclen < COLS-3) {
	       addstr(list[i].desc);
	       addch('\n');
	       free(list[i].desc);
	       i++;
	       continue;
	  }

	  /* move our progress meter to the beginning of the description */
	  laststop = list[i].desc;

	  /* loop until we determine it's the end and break */
	  while(1) {

	       progress = laststop+(COLS-3);

	       /* move our amount to copy back to a space */
	       while(progress[0] != ' ')
		    progress--;

	       /* copy in the intermediate characters */
	       strncpy(buffer, laststop, progress-laststop);
	       addstr(buffer);
	       addch('\n');

	       /* did that line involve hitting the bottom? */
	       getyx(stdscr, y, x);
	       if(y > LINES-3) {
		    key_pause();
		    clear();
	       }

	       /* if we just copied out the end of the description, we
		  can quit now */
	       if(strlen(buffer) < (COLS-3)) 
		    break;
		       
	       /* move out last stop up to just after the space */
	       laststop = progress-1;

	  } /* one more line out... */
	  
	  /* alright, we've printed it all */

	  free(list[i].desc);
	  i++;
	  
     } /* finished an entry... */

     /* alright, printed 'em all */

     free(list);
     key_pause();

     return;
}

void upload(void) {
     
     bbserr("Sorry, uploads are not implemented yet.\n");
     return;
}

void download(void) {

     bbserr("Sorry, downloads are not implemented yet.\n");
     return;
}

int enter_files(void) {

     FILE_Area *cur;
     int len;

     cur = file_arealist(&len);
     if(cur == NULL) {
          bbserr("Shit.  There don't seem to be any areas to send you to.\n");
          bbserr("Attempting to bail out...\n");
          return -1;
     }

     strcpy(this_node->filearea, cur[0].name);

     free(cur);

     return 0;

}
