
/* fsedit.c -- code for the full-screen message editor */

#include <stdio.h>
#include <stdlib.h>
#include <curses.h>

#include <common.h>
#include <client.h>

#include "fsedit.h"

static void fs_status(int x, int y, int ins);
static char *get_next_line(char **begin);
static int skip_line(char *line);

/* alright, a note on how the goddamn editor works.  it's really
   awkward at the moment, it's probably going to have to be totally
   rewritten at some stage of the game.

   x and y represent the position of the cursor on the screen...  x is
   always a valid way to find out how far along the line the cursor
   is. 

   topy is the line which is currently at the top of the window, to
   handle scrolling...   y+topy will always give the actual line that
   is being edited. */

char *fs_edit(char *existing, MSG_Head *qhead) {

     char *ret;
     int used = 0;
     int x = 0, y = 0, topy = 0;
     int i;
     int done;
     int ins = 0;
     WINDOW *edit;


     char *exwork;
     char *oldwork;
     char *lines[1024]; /* maximum edit buffer size is 1k lines.  I
			   hope this won't really be a problem. */


     if(existing != NULL) {
	  char *tmpline;

	  /* if there's supposed to be something in the edit buffer
	     already, copy it in. */
	  
	  exwork = (char *)alloca(strlen(existing)+1);
	  strcpy(exwork, existing);
	  oldwork = exwork;

	  /* add the quoting header. */
	  used = 2;
	  lines[0] = (char *)calloc(1, COLS);
	  lines[1] = (char *)calloc(1, COLS);
	  sprintf(lines[0], "On %s, %s wrote:", qhead->date, qhead->from);

/* we've chosen a \377 to be the character which signifies end of line here. */

	  while((tmpline = get_next_line(&exwork)) != NULL) {
	      
	       /* empty line? */
	       if(tmpline[0] == 0) {
		    lines[used] = calloc(sizeof(char), 1);
		    used++;
		    oldwork = exwork;
		    continue;
	       }

	       /* check to see if this line's got a control character
		  to worry about. */
	       if(skip_line(tmpline)) {
		    oldwork = exwork;
		    continue;
	       }

	       if(strlen(tmpline) <= COLS-5) {
		    /* the line in question is less than the screen
		       width we're working with, so we can move it all
		       at once. */

		    lines[used] = (char *)calloc(1, COLS);
		    sprintf(lines[used], "%c> %s", qhead->from[0], tmpline);
		  
	       } else {
		    /* word wrap when the next line is simply too
		       long.  attempting to actually precompensate
		       wordadj produces bugs.  damn...  but this
		       routine does work, but I expect it to crash in
		       the near future.  */

		    int copied = 0;
		    /* precompensate wordadj for the quote prefix */
		    int wordadj = 5;

		    /* remove the null terminator, it's meaningless */
		    tmpline[strlen(tmpline)] = '\r';

		    lines[used] = (char *)calloc(1, COLS);
		    /* figure out where the last word ended... */

		    while(tmpline[COLS-wordadj] != ' ') 
			 wordadj++;
		    
		    sprintf(lines[used], "%c> ", qhead->from[0]);
		    memcpy(lines[used]+3, tmpline, COLS-wordadj);
		    copied = (COLS - wordadj);

		    exwork = tmpline + copied;

		    /* we don't want to have a bunch of spaces
		       starting the next line that were left over
		       after that, so move exwork up until we're past
		       any of them. */
		    while(exwork[0] == ' ')
			 exwork++;
		    
	       } /* end of if...else.. (for line length) */

	       used++;
	       oldwork = exwork;
	  } /* ends while loop for copy. */
     } /* end of existing != NULL test */

     lines[used] = NULL;

     clear();

     if((edit = newwin(LINES-3, COLS, 2, 0)) == NULL) {
	  addstr("Failed to create editor window.\n");
	  return NULL;
     }

     immedok(edit, FALSE);
     clearok(edit, TRUE);
     scrollok(edit, FALSE);
     keypad(edit, TRUE);
     syncok(edit, TRUE); 

     scrollok(stdscr, FALSE);
     immedok(stdscr, FALSE);

     clear();

     mvprintw(0, 0, "OxidusEdit: /S=Save /A=Abort");
     
     /* draw a seperator bar under the instructions... */
 
     i = 0;
     while(i < COLS) { 
	  mvaddch(1, i, '-');
	  i++;
     }
     
     /* and at the bottom */

     i = 0;
     while(i < COLS) { 
	  mvaddch(LINES-1, i, '-');
	  i++;
     }
     werase(edit);
     
     /* draw in the stuff which came before, if used != 0 */
     if(used != 0) {
	  int iy = 0;
	  while((iy < used) && (iy < LINES-3)) {
	       mvwaddstr(edit, iy, 0, lines[iy]);
	       iy++;
	  }
     } else /* if used is 0, allocate ram for line 0. */ 
	  lines[0] = (char *)calloc(1, COLS);

     wmove(edit, 0, 0);
     x = 0;
     y = 0;
     cbreak();
     noecho(); 
     
     /* alright, enter the big editing loop. */
     
     done = 0;
     while(done == 0) {
	  int key;

	  /* deal with x moving off of the screen */
	  if(x >= COLS-1) {
	       y++;
	       x = 0;
	  }

	  if(x < 0) {
	       if(y == 0)
		    x = 0;
	       else {
		    y--;
		    x = strlen(lines[y+topy]);
	       }
	  }

	  /* deal with the user's having moved the cursor above the
	     top of the screen */
	  if(y < 0) {
	       y = 0;
	       if(topy != 0) {
		    int iy;

		    wclear(edit);
		    topy--;
		    iy = topy;
		    
		    while((iy < topy+(LINES-3)) && (lines[iy] != NULL)) {
			 mvwaddstr(edit, iy-topy, 0, lines[iy]);
			 iy++;
		    }
		    
	       } /* end of if(topy...) */
	  }

	  /* alright, we've moved off the bottom of the screen, scroll
	     and redraw. */
	  if(y >= LINES-3) {
	       int iy;
	       
	       y = 0;
	       wclear(edit);
	       
	       topy++;
	       iy = topy;
	       
	       while(iy < (LINES-3)+topy) {
		    if(lines[iy] == NULL)
			 break;

		    mvwaddstr(edit, y, 0, lines[iy]);
		    iy++;
		    y++;
	       }
	       y = LINES-4;
	  }
	  
	  /* we've moved down onto a line we've never touched before,
	     so allocate memory for it and increment used. */
	  if(y+topy > used) {
	       used = y+topy;
	       lines[used] = (char *)calloc(1, COLS);
	       lines[used+1] = NULL;
	  }

	  /* redraw status line */
	  fs_status(x,y+topy, ins);

	  /* update all the windows, as we're ready for them to press
	     a key */

	  wnoutrefresh(stdscr);
	  wnoutrefresh(edit);
	  wmove(edit, y, x);
	  doupdate();
	  
	  key = wgetch(edit);
	  
	  switch(key) {
	  case KEY_UP: 
	       y--;
	       break;
	  case KEY_DOWN:
	       y++;
	       break;
	  case KEY_LEFT:
	       x--;
	       break;
	  case KEY_RIGHT:
	       x++;
	       break;
	  case KEY_HOME:
	       x = 0;
	       break;
	       /* pgdn/pgup */
	  case KEY_NPAGE:
	       if(topy+(LINES-4) > used)
		    topy = used-2;
	       else
		    topy += LINES-5;
	       y = -1;
	       break;
	  case KEY_PPAGE:
	       if(topy-(LINES-4) < 0) 
		    topy = 0;
	       else
		    topy -= (LINES-3);
	       y = -1;
	       /* refreshing is broken, for some reason, so we'll push
		  a ctrl-l onto the input queue to cause the screen to
		  redraw. */
	       ungetch(0x0c);
	       break;
	  case KEY_BACKSPACE:
	  case KEY_DC:
	       /* hardcode in ^h and ^? as known delete characters. */
	  case 0x8:
	  case 0x7f:
	  {
	       int linein;
	       
	       x--;
	       linein = x;
	       /* loop and move every character ahead back by one. */
	       while(linein < strlen(lines[y+topy])) {
		    lines[y+topy][linein] = lines[y+topy][linein+1];
		    linein++;
	       }
	       /* set up the end of the line, for sure, and overwrite
		  the final character on the screen, which was removed. */
	       lines[y+topy][linein] = 0;
	       mvwaddstr(edit, y, 0, lines[y+topy]);
	       mvwaddch(edit,y,linein,' ');

	  }
	  break;
	  /* redraw the screen on a ctrl-l. in this case, we'll want a
	     complete repaint so we know what the lines array looks
	     like. */
	  case 0x0c:
	       refresh();
	       wclear(edit);
	       {
		    int iy;
		    
		    iy = topy;
		    
		    while((iy < topy+(LINES-3)) && (lines[iy] != NULL)) {
			 mvwaddstr(edit, iy-topy, 0, lines[iy]);
			 iy++;
		    }
	       }
	       break;

	       /* deal with coming in/out of insert mode, and hardcode
		  a ctrl-v to insert toggle. */
	  case KEY_IC:
	  case KEY_EIC:
	  case 0x16:
	       if(ins == 0)
		    ins = 1;
	       else
		    ins = 0;
	       break;

	       /* add support for deleting a single line from a
		  message. hardcode ctrl-k to do this, as well. */
	  case KEY_DL:
	  case 0x0b:
	  {
	       int iy;
	       iy = y+topy;
	       
	       free(lines[iy]);

	       while(iy < used) {
		    lines[iy] = lines[iy+1];
		    if(iy-topy < LINES-3) {
			 wmove(edit, iy-topy, 0);
			 wclrtoeol(edit);
			 if(lines[iy] != NULL)
			      waddstr(edit, lines[iy]);
		    }
		    iy++;
	       }
	       used--;
	  }
	  break;

	  case KEY_ENTER:
	  case '\n':
	  case '\r':
	       y++;
	       x = 0;
	       break;
	       
	  /* alright, they typed a /...  if we're at the beginning of
	     the line, it's meaningful. */
	  case '/':
	       if(x == 0) {
		    int nextkey;
		    
		    nextkey = wgetch(edit);
		    if((nextkey == 's') || (nextkey == 'S'))
			 done = 1;
		    else if((nextkey == 'a') || (nextkey == 'A'))
			 done = 2;
		    else {
			 mvwaddch(edit, y, x, key);
			 lines[y+topy][x] = key;
			 mvwaddch(edit, y, x+1, nextkey);
			 lines[y+topy][x+1] = nextkey;
			 x += 2;
		    }
	       } else {
		    mvwaddch(edit, y, x, key);
		    lines[y+topy][x] = (char)key;
		    x++;
	       }
	       
	       break;
	       
	  default:
	       mvwaddch(edit, y, x, key);
	       lines[y+topy][x] = (char)key;
	       x++;
	       break;
	       
	  } /* ends switch bit.. */

     } /* ends while...  what's done now? */

     /* bring the stuff we've dicked with that affects stdscr back
	into some sane values. */

     scrollok(stdscr, TRUE);
     immedok(stdscr, TRUE);
     nocbreak();
     echo();

     delwin(edit); 

     if(done == 1) {
	  /* done == 1, so we're saving the message. */
	  
	  int iy = 0;
	  char *tmpretbuf;
	  
	  /* set up tmpretbuf to be the maximum possible amount of
	     data we've got...   used*COLS. */
	  tmpretbuf = (char *)calloc(1, (used+1)*COLS);

	  while(lines[iy] != NULL) {
	       /* none of the lines have newlines on them anymore,
		  they were stripped off by the magic of strsep(). 
		  put on a \r, as that seems to be the line terminator
		  of choice for these messages.*/

	       strcat(tmpretbuf, lines[iy]); 
	       strcat(tmpretbuf, "\r");
 	       free(lines[iy]);
	       iy++;
	  }
	  
	  /* alright, now.  We'll allocate strlen+ bytes for the real
	     return buffer, and memcpy strlen in, throwing a null on
	     the end for good measure. */
	  
	  ret = (char *)malloc(strlen(tmpretbuf)+1);
	  memcpy(ret, tmpretbuf, strlen(tmpretbuf));
	  ret[strlen(tmpretbuf)] = 0;
	  free(tmpretbuf);
	  
     }

     /* done == 2, so ret should be NULL */
     if(done == 2) {
	  int iy = 0;
	  
/*	  while(lines[iy] != NULL) {
	       free(lines[iy]);
	       iy++;
	       } */
	  
	  ret = NULL;
     }


     return ret;

}     

/* update the status line, with the cursor position, and the insert
   status. */

void fs_status(int x, int y, int ins) {
     
     if(ins == 0)
	  mvwprintw(stdscr, LINES-1, 0, "--- (%d,%d) Ovr " ,x, y);
     else
	  mvwprintw(stdscr, LINES-1, 0, "--- (%d,%d) Ins " ,x, y);

     return;

} 

/* get_next_line fishes out the next "line" (up to another \r) from
   the body of text at begin, trying to be as nice to the text as a
   whole as possible.  strsep doesn't behave quite how we'd like here,
   and it causes us to stop quoting too early */

char *get_next_line(char **begin) {

     char *cut;
     char *ret;

     /* are we at the end? (next byte is a 0) */
     if((*begin)[0] == 0)
	  return NULL;

     /* possible brokenness if the next thing we see is a \r, which
	could be an issue on consecutive newlines.  handle here */

     if((*begin)[0] == '\r') {
	  ret = (*begin);
	  (*begin)++;
	  ret[0] = 0;
	  return ret;
     }

     /* look for a \r.  if we don't find one, but we didn't bail as we
	did above, then we've got a little more body without another
	newline. */
     if((cut = strchr((*begin), '\r')) == NULL) {
	  /* no more \r, more text */
	  ret = (*begin);
	  /* move up to that null char */
	  (*begin) += strlen((*begin));
	  return ret;
     }

     /* null terminate this section, and then move the beginning past
	the \r turned null */
     
     ret = (*begin);
     cut[0] = 0;
     (*begin) = cut+1;
    
     return ret;


}

/* check to see if this line has a control prefix, or is an
   AREA:/SEEN-BY: */

int skip_line(char *line) {

     /* if the first character's not 0x01, 'S' or 'A', we're good */
     if((line[0] != 0x01) && (line[0] != 'S') && (line[0] != 'A'))
	  return 0;

     /* if it's 0x01, leave now */
     if(line[0] == 0x01) 
	  return 1;

     
     if(!strncmp(line, "AREA", 4))
	  return 1;

     if(!strncmp(line, "SEEN-BY", 7))
	  return 1;

     /* welp, nothing matched */
     return 0;

}
