#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include <curses.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <errno.h>


#include <common.h>
#include <client.h>

#include "internode.h"
#include "disp.h"

extern BBS_Config *config;
extern BBS_Node *this_node;
extern int my_node;

/* who's online?  go through the nodes and see which ones have someone
   logged in. */

void who(void){

     BBS_Node *other_node;
     int nodenum;

     nodenum = 1;
 
     /* pretty header... */
     
     addch('\n');
     printw("%3s %20s %50s\n","#","Handle","Action");
     addstr("--- -------------------- --------------------------------------------------\n");
     addstr("(a * denotes unavailable for chat)\n\n");

     /* cycle each node individually */
     while(nodenum <= config->nodes) {
	  other_node = read_node(nodenum);
	  if(other_node != NULL) {
	       /* denying messages? */
	       if(other_node->avail < 2) 
		    addch('*');
	       printw("%2d %s\n", nodenum, other_node->what);
	       free(other_node);
	  }
	  nodenum++;
     }
     
	  
}


/* sends out a 1-liner to another node. */

void talk_to_node(void) {
     
     int db_sock, len;
     char buffer[256];

     int dest;
     BBS_Inc message;
     
     /* who do they want to talk to? */
     prompt("Destination node? ");

     getstr(buffer);
     message.dest_node = atoi(buffer);
     if(message.dest_node == 0) {
	  bbserr("Invalid node!\n");
	  return;
     }

     /* what to say? */
     prompt("Message: ");
     getstr(message.msg);
     addch('\n');
     
     /* build a node_msg to transmit */
     sprintf(buffer, "[%s (%d)]: ", this_node->current_user->handle, my_node);
     sprintf(message.msg, "%s", strcat(buffer, message.msg));
     message.orig_node = my_node;
     message.type[0] = '2';     
     message.type[1] = 0;

     /* send it */
     if(msg_transmit(&message) == -1) 
	  bbserr("Transmission failed.\n");

     return;
}

/* attempt to initiate chat with another node...  */

void want_chat(void){

     BBS_Inc message;
     char buffer[80];
     BBS_Node *destnode;
     int dest;
     int port;
     int tries;
     struct sockaddr_in chaddr;
     int chsock, conn;
     int flags;
     int socklen = sizeof(struct sockaddr_in);
     int n;

     /* figure out who to chat with */
     prompt("Try to chat with which node [0 == abort]? ");
     getstr(buffer);
     dest = atoi(buffer);
     if(dest == 0) {
	  bbserr("Aborting chat request.\n");
	  return;
     }
     
     /* make sure their destination exists */
     if((destnode = read_node(dest)) == NULL) {
	  bbserr("Invalid node.\n");
	  return;
     }

     /* and will they take a chat?  override if we're a sysop. */
     if(((destnode->avail < 2) && (this_node->current_user->access <
				   65000)) || (destnode->avail < 1)) {
	  bbserr("Sorry, that node is not available for chat.\n");
	  free(destnode);
	  return;
     }

     free(destnode);

     /* set up a socket so we they can connect to us and begin
	communication. */

     port = 10040+my_node;

     chsock = socket(AF_INET, SOCK_STREAM, 0);
     
     flags = fcntl(chsock, F_GETFL, 0);
     fcntl(chsock, F_SETFL, O_NONBLOCK |flags);

     chaddr.sin_family = AF_INET;
     chaddr.sin_port = htons(port);
     chaddr.sin_addr.s_addr = inet_addr(config->db_ip);

     n = 1;
     if (setsockopt(chsock, SOL_SOCKET, SO_REUSEADDR, &n,
                    sizeof(n))<0) {
          addstr("Failed to set socket up properly.\n");
          return;
     }

     /* cycle through a few ports to find one that's not a problem for
	us to bind to. */

     tries = 0;
     while(tries < 5) {
	  if(bind(chsock, (struct sockaddr *)&chaddr,
		  sizeof(struct sockaddr_in)) != 0) { 
	       tries++;
	       port++;
	       chaddr.sin_port = htons(port);
	  }
	  else
	       tries = 20;
     }

     /* well, looks like we didn't get a port.  damn. */
     if(tries != 20) {
	  bbserr("Couldn't establish communications socket.\n");
	  switch(errno) {
	  case EBADF:
	       bbserr("Invalid FD for socket.\n");
	       break;
	  case EINVAL:
	       bbserr("Socket already bound.\n");
	       break;
	  case EACCES:
	       bbserr("Not permitted to bind.\n");
	       break;
	  case ENOTSOCK:
	       bbserr("Tried to bind to something not a socket.\n");
	       break;
	  default:
	       bbserr("Unknown error.\n");
	  }
	  close(chsock);
	  return;
     }
	  

     /* set up the message we're transmitting through the server to
	tell the other node we want them. */

     message.dest_node = dest;
     message.orig_node = my_node;
     message.start_time = time(NULL);
     sprintf(message.type, "3");
     sprintf(message.msg, "%d %d", my_node, port);
     msg_transmit(&message);

     /* message sent to the server, begin listening. */
     if(listen(chsock, 3) != 0) {
	  printw("Couldn't listen for the other node.\n");
	  close(chsock);
	  return;
     }

     /* try to accept the connection for a few seconds, and bail out
	if they never answered. */

     tries = 1;
     addstr("Waiting for response, press any key to abort..");
     while (tries < 21) {
	  printw("%d..",tries);
	  if((conn = accept(chsock, (struct sockaddr *)&chsock,
			    &socklen)) == -1) { 
	       tries++;
	       halfdelay(10);
	       if(getch() != ERR) {
		    tries = 30;
		    break;
	       }
	  }
	  else
	       break;
     }
     addch('\n');
     
     if(tries > 20) {
	  bbserr("No response from destination node at all.\n");
	  close(chsock);
	  return;
     }
     
     /* they answered, see if it was with a no. */
     read(conn, buffer, 1);
     if(buffer[0] != 'y') {
	  bbserr("They refused our request.\n");
	  shutdown(conn, 2);
	  close(conn);
	  close(chsock);
	  return;
     }
     
     /* I guess they like us.  Pass the FD of our connection down to
	the bbs module. */

     fs_chat(conn);

     /* clean up and get out. */
     close(conn);
     close(chsock);
     return;
     
}

/* routine to send off a node_msg to the server */

int msg_transmit(BBS_Inc *message) {

     int db_sock, len;
     char buffer[80];
     DB_Req req;
     int ret;

     req.req = INC_SEND;
     req.datasize = sizeof(BBS_Inc);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, message, sizeof(BBS_Inc));

     if(db_reply(db_sock) == -1)
	  ret = -1;
     else
	  ret = 0;

     shutdown(db_sock, 2);
     close(db_sock);

     return ret;
     
}

/* someone asked us to chat with them, deal with that. */

void inbound_chat(int node_req, int port) {

     char opt;
     int chsock;
     int len;
     char buffer[256];
     struct sockaddr_in chaddr;

     printw("Inbound chat request from node %d. Accept [y/n]? ");
     
     opt = 0;
     while((opt != 'y') && (opt != 'n')) {
	  cbreak();
	  opt = getch();
	  nocbreak();
     }

     /* they told us what to do, now connect to the advertised port
	and tell the requesting node what we've decided. */

     chaddr.sin_family = AF_INET;
     chaddr.sin_port = htons(port);
     /* fix for distributed nodes sometime. */
     chaddr.sin_addr.s_addr = inet_addr(config->db_ip);
     
     chsock = socket(AF_INET, SOCK_STREAM, 0);

     if(connect(chsock, &chaddr, sizeof(chaddr)) == -1) {
          bbserr("Welp, they don't seem to be waiting for us anymore.\n");
          return;
     }

     write(chsock, &opt, 1);
     if(opt == 'n') {
	  len = read(chsock, buffer, 256);
	  close(chsock);
	  addch('\n');
	  return;
     }

     fs_chat(chsock);
     close(chsock);
     return;
     
}

void fs_chat(int sock){
     
     char in, out;
     int len;
     WINDOW *me;
     WINDOW *him;
     char buffer[256];
     int x, y;

     erase();

     me = newwin((LINES/2)-1, COLS, 0, 0);
     him = newwin((LINES/2)-1, COLS, LINES/2, 0);
     
     attron(A_REVERSE);
     mvwprintw(stdscr, (LINES/2)-1, 0, "Inter-node chat.  Press Ctrl-A to leave."); 
     attroff(A_REVERSE);

     immedok(me, TRUE);
     scrollok(me, TRUE);
     immedok(him, TRUE);
     scrollok(him, TRUE);

     /* if they've not typed something every 2/10's of a second, they
	see if the other node's got something to say. */
     halfdelay(2);

     in = ERR;
     while(in != 1) {
	  in = wgetch(me);

	  if(in == ERR)
	    in = 0;
	
	  write(sock, &in, 1);
	  
	  len = read(sock, buffer, 255);
	  if(len == -1) 
	       break;
	  else {
	  buffer[len] = 0;
	  if(buffer[0] == 1)
	       break;
	  if(buffer[0] != 0) 
	       waddstr(him, buffer);
	  }
     }


     delwin(me);
     delwin(him);
     shutdown(sock, 2);

     move(LINES-1, 0);
     addstr("Chat terminated.\n");
     return;
}
