#include <stdio.h>
#include <curses.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <sys/socket.h>

#include <common.h>
#include <client.h>

#include "msgs.h"
#include "fsedit.h"
#include "disp.h"

char *mkfidotime(void);

extern BBS_Config *config;
extern BBS_Node *this_node;
extern int my_node;

int *scan_personal(char *area, char *user, int range) {
     
     int db_sock;
     char buffer[80];
     int *ret;
     int x;
     MSG_Ref lookup;

     /* here we allocate an array much bigger than we likely need... */
     DB_Req req;
     int size;

     strcpy(lookup.name, area);

     req.req = MSG_SCAN_USER;
     req.datasize = sizeof(MSG_Ref)+20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, &lookup, sizeof(MSG_Ref));
     netprintf(db_sock, "%s", user);

     if((size = db_reply(db_sock)) == -1)
          return NULL;

     x = 0;
     ret = (int *)malloc(size);

     do {
	  read(db_sock, &(ret[x]), sizeof(int));
	  x++;
     } while (ret[x-1] != 0);


     close(db_sock);
     return ret;

}
     

/* and now, the user interface parts.  ick. */

int enter_messages(void) {
     
     MSG_Area *cur;

     cur = all_areas();
     if(cur == NULL) {
	  bbserr("Shit.  There don't seem to be any areas to send you to.\n");
	  bbserr("Attempting to bail out...\n");
	  return -1;
     }

     strcpy(this_node->area, cur[0].area_name);

     free(cur);

     this_node->cur = read_pointer(this_node->current_user->handle, 
				   this_node->area);

     return 0;

}
	  
/* display a list of message areas to the user */
void list_areas(void) {

     MSG_Area *cur;
     int x, y, i;
     char junk[5];

     i = 0;
     clear();
     move(0,0);
     getyx(stdscr, y, x);
     
     cur = all_areas();

     /* give them a header*/

     printw("%20s %50s\n", "Area", "Description");
     y++;
     addstr("-------------------- --------------------------------------------------\n");
     y++;
     do {
	  /* check to see if they can access this area */
	  if(cur[i].access <= this_node->current_user->access) { 
	       /* any unread messages for this person? */
	       if(cur[i].last >
		  read_pointer(this_node->current_user->handle,
			       cur[i].area_name)) 
		    addch('*');
	       else
		    addch(' ');

	       printw("%20s %50s\n", cur[i].area_name, cur[i].desc);
	       y++;
	  } 
	  i++;
	  
	  /* end of screen? */
	  if(y == LINES-2) {
	       addstr("Press return to continue...");
	       getstr(junk);
	       clear();
	       move(0,0);
	       getyx(stdscr, y,x);
	  }
     } while(cur[i].done != 1);

     addstr("Press return to continue...");
     getstr(junk);

     free(cur);

}

/* list the messages in a given area. */
void list_messages(void) {

     MSG_Head *cur;
     MSG_Area *area;
     int x, y;
     char start[10];

     area = read_area(this_node->area);
     if(area == NULL) {
	  bbserr("Internal failure -- Invalid message area.\n");
	  return;
     }

     printw("Begin listing at [%d-%d, ENTER=%d]: ", area->first,
	    area->last, this_node->cur);
     getstr(start);
     if((x = atoi(start)) == 0)
	  x = this_node->cur;
     
     if((x < area->first) || (x > area->last)) {
	  bbserr("Message out of range.\n");
	  free(area);
	  return NULL;
     }
     

     /* header... */
     printw("%3s %20s %20s %30s\n", "#", "From:", "To:", "Subject:");
     addstr("--- --------------------- ---------------------- -----------------------------\n\n");
     
     y = 4;

     /* attempt to read each header and display the necessary info
	about it. */

     while(x <= area->last) {
	  MSG_Ref lookup;
	  strcpy(lookup.name, this_node->area);
	  lookup.msg = x;

	  cur = get_head(&lookup);
	  /* make sure it's not a private message */
	  if((cur != NULL) || ((cur->attribute & MSG_PRIV) &&
			       (this_node->current_user->access < 65000))) {
	       printw("%3d %20s %20s %30s\n", x, cur->from, cur->to, cur->subject);
	       free(cur);
	  }

	  x++;
	  y++;
	  if(y == 23) {
	       /* scrolled past bottom.  why is that a hardcoded
		  value? */
	       key_pause();
	       clear();
	       move(0,0);
	       y = 1;
	  }
	  
     }
     
}

/* so, they actually want to view a message.  this stuff,
   incidentally, is quite ugly. */

void msg_view(MSG_Ref *lookup) {

     MSG_Area *this_area;
     MSG_Head head;
     char *body;
     int msg;

     body = NULL;
     
     if(lookup == NULL) {
	  bbserr("Internal mangled call to msg_view, no area.\n");
	  return;
     }

     if((this_area = read_area(lookup->name)) == NULL) {
	  printw("Internal mangled call to msg_view, invalid area.\n");
	  return;
     }

     msg = lookup->msg;
     
     /* if we weren't told which message to display, ask */
     if(msg == 0) {
	  prompt("Read message # [return = next]: ");
	  scanw("%d", &msg);
	  /* a non-decimal answer gets the next message */
	  if(msg == 0) {
	       this_node->cur++;
	       msg = this_node->cur;
	  }
     }

     msg--; /* need to be adequately prepped for this ugly do-while */

     /* get the next message.  and if they can't read it, keep going. */
     do {
	  msg++;
	  
	  if(body != NULL) {
	       free(body);
	       body = NULL;
	  }

	  /* we're at the end. */
	  if(msg > this_area->last) {
	       bbserr("No more accessable messages in this area.\n");
	       this_node->cur = this_area->last; 
	       free(this_area);
	       update_pointer(this_node->current_user->handle,
			      this_node->area, this_node->cur);
	       return;
	  }
	  
	  lookup->msg = msg;
	  body = read_message(lookup, &head);
	  
     } while(((head.attribute & MSG_PRIV) &&
	      (strcasecmp(this_node->current_user->handle,
			  head.to)) &&
	      (this_node->current_user->access < 65000)) ||
	     (body == NULL));
     /* my, what a large conditional you've got there...  all the
	better to confuse you with, by dear. */

     /* update pointers to the proper location. */
     this_node->cur = msg;
     update_pointer(this_node->current_user->handle,
		    this_node->area, this_node->cur);
    

     /* actually show the text */
     msg_disp(&head, body);
     free(this_area);
     free(body);


}

/* this is the one that actually puts the text of the message onto the
   screen.  I fear it.  It makes me understand why windows never
   evolves...  once you get bad code to finally stand on some kind of
   rickety supports, you're terrified to change anything. */

void msg_disp(MSG_Head *head, char *body) {

     char *line;
     int x,y;
     WINDOW *header;
     WINDOW *msg_text;
     int bodylen, linelen, parsed = 0;
     char *buffer;

     buffer = alloca(COLS+1);
     clear();

     /* set up the windows we're going to display the message in. */
     header = newwin(5, COLS, 0, 0);
     msg_text = newwin(LINES-5, COLS, 5, 0);
     scrollok(msg_text, TRUE);
     immedok(msg_text, TRUE);
     immedok(header, TRUE);

     wattron(header, A_REVERSE);

     /* display the message header in the proper window, differently
	for local or network messages. */

     if(head->attribute & MSG_LOCAL) {
          wprintw(header, "   From: %s", head->from);
	  right_justify(header, " ");
          wprintw(header, "     To: %s", head->to);
	  sprintf(buffer, "Date: %s", head->date);
	  right_justify(header, buffer);
     } else {
          char addr[30];
	  tAddress lookup;
	  char *name;
	  
	  /* prepare to look up the name of the node. */
	  lookup.Zone = head->origzone;
	  lookup.Net = head->orignet;
	  lookup.Node = head->orignode;
	  lookup.Point = head->origpoint;
	
          wprintw(header, "   From: %s", head->from);

	  /* try to lookup the name for this node, and display it in
	     the header if they've got one. */

	  if((name = name_nl2k(&lookup)) != NULL) {
	       sprintf(addr, "%.28s (%d:%d/%d.%d)",
		       name, head->origzone,
		       head->orignet, head->orignode,
		       head->origpoint);
	       free(name);
	       } else /* no name here, only display address. */
	  sprintf(addr, "(%d:%d/%d.%d)", head->origzone, head->orignet,
		  head->orignode, head->origpoint);
	  
	  right_justify(header, addr);
          wprintw(header, "     To: %s", head->to);
	  sprintf(buffer, "Date: %s", head->date);
	  right_justify(header, buffer);

     }
     
     /* is there a subject at all? */
     if (head->subject[0]) {
          wprintw(header, "Subject: %s", head->subject);
          right_justify(header," ");
     }
     
     /* flags from the header. */
     waddstr(header, "  Flags: ");
     if(head->attribute & MSG_LOCAL)
          waddstr(header, "Local ");
     if(head->attribute & MSG_PRIV)
          waddstr(header, "Pvt ");
     if(head->attribute & MSG_RECD)
          waddstr(header, "Recv ");
     if(head->attribute & MSG_SENT)
          waddstr(header, "Sent ");
     if(head->attribute & MSG_TRANSIT)
          waddstr(header, "Transit ");
     if(head->attribute & MSG_ATTACH)
          waddstr(header, "Attach ");
     right_justify(header," ");    

     wattroff(header, A_REVERSE);

     /* now do the display of the message body */
     wmove(msg_text, 0, 0);
     bodylen = strlen(body);
     line = body; 

     /* convert all \r's to nulls.  this should probably seperately
	handle \r, \n, or \r\n...  \r seems to be most common in fido,
	but it's by no means universal... */

     while((line=strchr(line+1,'\r')))
          *line = '\0';

     buffer[COLS-1] = '\0';

     parsed = 0;

     while(parsed < bodylen) {

            /* How ugly art thou? Let me count the ways... Ok, what this
               does is make sure that we're at the beginning of a line of
               text, because we could _theoretically_ have a 'AREA:' or
               'SEEN-BY' in the body of a message at the parsed+COLS+1
               position. */

            if((!parsed || !(body[parsed-1])) && ((*(body+parsed) == 0x01)
                  || !strncmp(body+parsed, "AREA:",5)
                  || !strncmp(body+parsed, "SEEN-BY",7))) {
                     parsed += strlen(body+parsed)+1;
                     continue;
            }

            getyx(msg_text, y, x); 

              if ((linelen = strlen(body+parsed)) < COLS-1) {
                wprintw(msg_text, "%s\n",body+parsed);
                parsed += linelen+1;
              } else {
                strncpy(buffer,body+parsed, COLS-1);
                wprintw(msg_text, "%s\n",buffer);
                parsed += COLS-1;
                if (linelen == COLS-1) parsed++;
              }

	      /* end of screen, prompt. */
          if (y >= LINES-8 && parsed < bodylen) {
               wprintw(msg_text, "\nPress any key to continue...");
               cbreak();
               noecho();
               getch();
               echo();
               nocbreak();
               werase(msg_text);
          }
     }

     /* finished up with the display */
     wprintw(msg_text, "\nPress any key to continue...");
     cbreak();
     noecho();
     getch();
     echo();
     nocbreak();

     delwin(header);
     delwin(msg_text);
     clear();

}

/* not only do they want to read the messages, the impudent bastards
   even want to write them. */
void compose_message(char *dest, MSG_Ref *original) {

     MSG_Area *this_area;
     MSG_Net *net;
     char buffer[80];
     FILE *fp;
     char exec[80];
     char file[80];
     char *body;
     MSG_Head head, rhead;
     char opt;
     char *timestr;
     time_t secs;
     int valid;
     char *rbody = NULL;
     MSG_Ref where;
     
     /* get the area and net config with where they're supposed to post to. */

     this_area = read_area(this_node->area);
     net = read_net_num(this_area->network_num);
     if((this_area == NULL) || (net == NULL)) {
	  /* either the area or the net was invalid.  In theory, we
	     should never get here.  */
	  bbserr("Area or network invalid in compose_message.\n");
	  bbserr("Area: %x, Net: %x\n", this_area, net);
	  bbserr("Something's mangled in the message areas.  Sorry.\n");
	  return;
     }

     /* do the initial preperation of the message header. */

     head.attribute = 0;

     if(this_area->real == 0) 
	  strcpy(head.from, this_node->current_user->handle);
     else
	  sprintf(head.from, "%s %s", this_node->current_user->first_name,
		  this_node->current_user->last_name);

     timestr = mkfidotime();
     strcpy(head.date, timestr);
     
     /* and for clarity, we'll tell the user what we're doing here so
	they they'll see if something's wrong */
     printw("From: %s", head.from);

     /* network message?  we've got more to do if so. */
     if(this_area->network_num < 1) {
	  head.attribute = head.attribute | MSG_LOCAL;
	  addch('\n');
     }
     else {
	  head.origzone = net->zone;
	  head.orignet = net->net;
	  head.orignode = net->node;
	  head.origpoint = net->point;
	  head.attribute |= MSG_TRANSIT;
	  printw(" (%d:%d/%d.%d)\n",head.origzone, head.orignet, head.orignode, head.origpoint);
     }
     
     /* determine if this area is set as private, and then turn on the
	bit if it is.  don't allow posting of private messages in
	non-private areas, it's stupid. */

     if(this_area->private == 1) 
	  head.attribute = head.attribute | MSG_PRIV;
     
     /* the following code is both stupid and terrifying.  it
	basically makes sure the user's not fucking up when they
	decide who to send their message to. */

     valid = 0;
       
     if(original->msg == 0) {
	  if(dest == NULL) {
	       while(valid == 0) {
		    
		    /* alright, they've got to tell us who it's to, no
		       matter what. */
		    
		    prompt("To: ");
		    scanw("%s", head.to);
		    /* didn't they tell us?  quit if not. */
		    if(strlen(head.to) < 2)
			 return;
		    
		    /* a private network area means netmail.  we need
		       to know the destination. */
		    if((this_area->network_num != 0) && (head.attribute &
							 MSG_PRIV)) {
			 prompt("Network address: ");
			 /* set everything to 0 as a precaution... */
			 head.destzone = 0;
			 head.destnet = 0;
			 head.destnode = 0;
			 head.destpoint = 0;
			 scanw("%hd:%hd/%hd.%hd", &(head.destzone), 
			       &(head.destnet),
			       &(head.destnode), &(head.destpoint));
			 
			 printw("To: %s (%d:%d/%d.%d)\n", head.to,
				head.destzone, head.destnet,
				head.destnode,  head.destpoint);
			 break;
			 
		    } else if(head.attribute & MSG_PRIV) {
			 /* local message, so make sure they're trying
			    to send this to a real user. */
			 /* LEAK */
			 if(read_user(head.to) == NULL) {
			      valid = 0;
			      printw("Invalid local user.\n");
			 } else
			      valid = 1;
		    }
		    else
			 valid = 1;
		    
	       } /* ends the validation while loop, we can proceed if
		    they've done the right thing */
	       	       
	  } else { /* not a reply, but dest != NULL...  stuff like
		      sysop feedback would trigger this. */ 
	       if(this_area->network_num == 0) { /* local message */
		    head.destzone = 0;
		    head.destnet = 0;
		    head.destnode = 0;
		    head.destpoint = 0;
		    strcpy(head.to, dest);
		    /* LEAK */
		    if(read_user(head.to) == NULL) {
			 /* directed to post to a fake user....  hmm. */
			 bbserr("Somehow, the message system has decided to make you post to a user\n");
			 bbserr("that doesn't exist in a local area.  I'm not going to play that game.\n");
			 return;
		    }
	       } else { /* alright, not local...  we've got to break the
			 destination out of the format which we passed
			 it as as a string. */
		    sscanf(dest, "%s+%hd:%hd/%hd.%hd", head.to, 
			   &(head.destzone), &(head.destnet), 
			   &(head.destnode), &(head.destpoint));
		    head.attribute |= MSG_TRANSIT;
	       }
	  } /* ends the else condition for dest != NULL */
     
	  /* not a reply here, let's ask for the subject */

	  prompt("Subject: ");
	  getstr(head.subject);
	  
	  addch('\n');
	  
     } else { /* reply != 0 here */
	  /* so, prepare eveything that needs to be set up in the case
	     of a reply...  proper addressing, quotes, all that sort
	     of shit.  Quoting is broken, it doesn't split lines
	     correctly.  the full screen editor tries to compensate
	     for this. */
	  
	  char *line;
	  char *reptxt;
	  static char *repwork;

	  /* set up a header and get the text of the reply. */
	  reptxt = read_message(original, &rhead);
	  if(reptxt == NULL) {
	       printw("uhm, odd, I've got you replying to a non-existant message.\n");
	       return;
	  }

	  /* prep the hadrer of the new message fully. */
	  strcpy(head.subject, rhead.subject);
	  strcpy(head.to, rhead.from);
	  head.destzone = rhead.origzone;
	  head.destnet = rhead.orignet;
	  head.destnode = rhead.orignode;
	  head.destpoint = rhead.origpoint;
	  
	  /* move the body of the message elsewhere.  the actual
	     reason for this escapes my mind at the moment. */

	  repwork = (char *)malloc(strlen(reptxt)+1);
	  strcpy(repwork, reptxt);
	  free(reptxt);
	  
	  /* allocate the ram for the reply body.  go for...  the size
	     of the reply text, plus, say, 100 bytes for the tagline,
	     and, maybe, (strlen(reptxt) / 40) *6, as an arbitrary
	     amount to be used for the line prefixes. */

/*	  rbody = (char *)malloc(strlen(repwork)+100+ 
	  ((strlen(repwork)/40)*6)); */

	  /* strip out control
	     lines here */

/*	  while((line = strsep(&repwork, "\r")) != NULL)
               if((line[0] != 0x01) && (line != strstr(line, "AREA:"))
                  && (line != strstr(line, "SEEN-BY"))) {
		    
		    char workline[COLS];

		    sprintf(workline, "%s\n", line);
		    strcat(rbody, workline);
	       }
*/
/* 	  strcpy(rbody, repwork);

	  free(repwork); */

	  rbody = repwork;
	  
	  /* alright, now rbody is complete with the stuff which needs
	     shoved in for our reply. */
     
     } /* we've figured out if we've got a reply or not, and
	  everything's prepped. */
     
     /* if it's local, turn the bit on */
     if(this_area->network_num == 0) 
	  head.attribute = head.attribute | MSG_LOCAL;

     /* alright.  theoretically, we've set up the headers, and prepped
	anything that needs to be done for a reply (e.g. quoted
	text).  so, pass them off into the full screen editor, and
	hope they make it back.... :) */

     if((body = fs_edit(rbody, &rhead)) != NULL) {
	  /* alright, there's a buffer after the full screen editor! */

	  printw("To: %s (%d:%d/%d.%d)\n", head.to, head.destzone, 
		 head.destnet, head.destnode,  head.destpoint);

	  printw("Sending %d length message.\n",strlen(body));
	  strcpy(where.name, this_area->area_name);
	  write_message(&where, &head, body);
	  free(body);
     }
     else
	  addstr("Message discarded.\n");

     return;

}

/* allows someone to delete a message.. */    
void erase_message(int msg) {

     MSG_Head *head;
     char *body;
     MSG_Ref lookup;

     /* ask which... */
     if(msg == 0) {
	  prompt("Erase which message? ");
	  scanw("%d", &msg);
     }

     /* non-decimal reply == abort */
     if(msg == 0) 
	  return;

     strcpy(lookup.name, this_node->area);
     lookup.msg = msg;

     /* try to read the message to determine it's existance */
     head = get_head(&lookup);
     if(head == NULL) {
	  bbserr("No such message.\n");
	  return;
     }

     /* ensure that they're cleared to delete it.  either the
	recipient or the sysop can do this. */

     if(strcasecmp(head->to, this_node->current_user->handle) &&
	(this_node->current_user->access < 65000)) {
	  bbserr("That message isn't addressed to you!\n");
	  free(head);
	  return;
     }

     free(head);

     clear_message(&lookup);
     
}

/* scan for new messages to the user. */
void check_messages(void) {

     MSG_Area *area;
     int *msgs;
     int x;
     char opt;


     area = read_area(this_node->area);
     if(area == NULL) {
	  bbserr("Fatal error scanning for messages.\n");
	  bbserr("Attempted to scan invalid area.\n");
	  return;
     }

     msgs = scan_personal(area->area_name,
			  this_node->current_user->handle, area->last
			  - area->first);

     /* tell them what the server found. */

     x = 0;
     addstr("Messages for you: ");
     while(msgs[x] != 0) {
	  printw("%d ", msgs[x]);
	  x++;
     }

     free(msgs);
     
     /* if there weren't any, at least say something. */
     if(x == 0)
	  addstr("none ");
     
     printw("in area %s.\n", area->area_name);

     free(area);

}

/* switch to another message area. */
void change_area(char *name) {

     MSG_Area *area;

     /* does it exist? */
     if((area = read_area(name)) == NULL) {
	  bbserr("Invalid message area.\n");
	  return;
     }
     
     /* and are they allowed to be there? */
     if(area->access > this_node->current_user->access) {
	  bbserr("Invalid message area.\n");
          return;
     }
     
     strncpy(this_node->area, area->area_name, strlen(area->area_name)+1);
     this_node->cur = read_pointer(this_node->current_user->handle, 
				   area->area_name);

     free(area);

     return;

}

/* go to the next area. */ 

void next_area(void) {

     int db_sock, len;
     MSG_Area next;
     MSG_Area *cur;
     DB_Req req;
     int size;

     if((cur = read_area(this_node->area)) == NULL) {
	  bbserr("Something's wrong.  You're not in a valid area.\n");
	  return;
     }

     req.req = MSG_AREA_NEXT;
     req.datasize = sizeof(MSG_Area);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return;

     write(db_sock, cur, sizeof(MSG_Area));
     free(cur);

     if(db_reply(db_sock) == -1) {
	  bbserr("No more areas.\n");
	  close(db_sock);
	  return;
     }

     /* there's another area, so get the data for it */
     
     read(db_sock, &next, sizeof(MSG_Area));

     shutdown(db_sock, 2);
     close(db_sock);

     strcpy(this_node->area, next.area_name);
     /* update their node info and last read pointer */
     this_node->cur = read_pointer(this_node->current_user->handle,
                                   next.area_name);

     /* if they're not allowed to be there, go to the next one. */
/*     if(next.access > this_node->current_user->access) {
          next_area();
	  return;
     }
*/
     return;
     
}

/* like next area, but previous */
void prev_area(void) {

     int db_sock, len;
     MSG_Area next;
     MSG_Area *cur;
     DB_Req req;
     int size;

     if((cur = read_area(this_node->area)) == NULL) {
          bbserr("Something's wrong.  You're not in a valid area.\n");
          return;
     }

     req.req = MSG_AREA_PREV;
     req.datasize = sizeof(MSG_Area);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return;

     write(db_sock, cur, sizeof(MSG_Area));
     free(cur);


     if(db_reply(db_sock) == -1) {
          bbserr("No more areas.\n");
          close(db_sock);
          return;
     }

     /* there's another area, so get the data for it */

     read(db_sock, &next, sizeof(MSG_Area));

     shutdown(db_sock, 2);
     close(db_sock);

     strcpy(this_node->area, next.area_name);
     /* update their node info and last read pointer */
     this_node->cur = read_pointer(this_node->current_user->handle,
                                   next.area_name);

     /* if they're not allowed to be there, go to the next one. */
     if(next.access > this_node->current_user->access) {
          prev_area();
          return;
     }

     return;

}


/* this makes an ascii time which fidonet is happy with.  asctime()
   doesn't give us something that fits in 20 chars */

char *mkfidotime(void) {

     struct tm *now;
     time_t secs;
     static char *ret;
     char month[4];
     int year;

     /* fetch the structure for our time... */
     secs = time(NULL);
     now = localtime(&secs);
     
     /* make our month */
     switch(now->tm_mon) {
     case 1:
	  sprintf(month, "Jan");
	  break;
     case 2:
	  sprintf(month, "Feb");
	  break;
     case 3:
	  sprintf(month, "Mar");
	  break;
     case 4:
	  sprintf(month, "Apr");
	  break;
     case 5:
	  sprintf(month, "May");
	  break;
     case 6:
	  sprintf(month, "Jun");
	  break;
     case 7:
	  sprintf(month, "Jul");
	  break;
     case 8:
	  sprintf(month, "Aug");
	  break;
     case 9:
	  sprintf(month, "Sep");
	  break;
     case 10:
	  sprintf(month, "Oct");
	  break;
     case 11:
	  sprintf(month, "Nov");
	  break;
     case 12:
	  sprintf(month, "Dec");
	  break;
     default:
	  sprintf(month, "Err");
	  break;
     }
     
     /* fix up the year for 2000+ (tm_year == years since 1900) */
     if(now->tm_year >= 100)
	  year = now->tm_year-100;
     else
	  year = now->tm_year;

     /* make the string... */
     ret = (char *)malloc(20);
     
     sprintf(ret, "%.2d %s %.2d  %.2d:%.2d:%.2d", now->tm_mday, month,
	     year, now->tm_hour, now->tm_min, now->tm_sec);

     /* free(now); ah, fufc, how I hate thee...  */
     return ret;

}
