#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <bbsglob.h>

extern BBS_Config *config;

void netprintf(int sock, char *fmt, ...) {

     char buffer[256];
     va_list argptr;

     va_start(argptr, fmt);

     vsprintf(buffer, fmt, argptr);
     va_end(argptr);

     write(sock, buffer, strlen(buffer)+1); /* go +1, and make sure
					       that we transmit the
					       null terminator. */
     return;

}

char *netgets(int sock) {

     static char *ret;

     ret = (char *)calloc(1,256);

     if(read(sock, ret, 255) == 0) {
	  free(ret);
	  ret = NULL;
     }

     return ret;

}

int read_system_config(void)
{
     FILE *fp;
     char *field;
     char *value;
     char filename[80];
     char *env;
     char *fullline;
     char *fld_free;

     env = getenv("ox_config");
     if(env == NULL)
	  env = getenv("OX_CONFIG");
     if(env != NULL) 
	  sprintf(filename, "%s/config.ox", strsep(&env, "= "));
     else
	  /* assume we're one subdir lower than where the config lies */
	  sprintf(filename, "config/config.ox");

     /* not totally sure how the enviornment stuff works...  freeing
	this may mess up the **enviorn pointers in some way, I'll play
	it safe. */

/*      if(env != NULL) 
	free(env); */
     
     if ((fp = fopen(filename,"r")) == NULL) {
          fprintf(stderr,"error: couldn't read system config: %s\n",
                  filename);
          exit(127);
     }
     /* error check these fuckers eventually when we insert
	them. */
     
     fld_free = (char *)calloc(sizeof(char), 80);
     field = fld_free;
     
     while (fgets(field, 80, fp) != NULL) {
	  fullline = field;

	  field = strsep(&fullline, " ");
	  value = strsep(&fullline, "\n");

	  if(!strcmp(field,"name")) 
	       strncpy(config->system_name, value,
		       sizeof(config->system_name));
	  else if(!strcmp(field,"sysop")) 
	       strncpy(config->sysop, value,
		       sizeof(config->sysop));
	  else if(!strcmp(field,"logfile")) 
	       strncpy(config->logfile, value,
		       sizeof(config->logfile));
	  else if(!strcmp(field,"port")) 
	       config->port = atoi(value);
	  else if(!strcmp(field,"setuid")) 
	       config->setuid = atoi(value);
	  else if(!strcmp(field,"setgid")) 
	       config->setgid = atoi(value);
	  else if(!strcmp(field,"phone"))
	       strncpy(config->phone, value,
		       sizeof(config->phone));
	  else if(!strcmp(field,"sysbase")) 
	       strncpy(config->base_dir, value,
		       sizeof(config->base_dir));
	  else if(!strcmp(field,"issue")) 
	       strncpy(config->issue, value,
		       sizeof(config->issue));
	  else if(!strcmp(field,"motd"))	     
	       strncpy(config->motd, value,
		       sizeof(config->motd));
	  else if(!strcmp(field,"logout")) 
	       strncpy(config->logout, value,
		       sizeof(config->logout));
	  else if(!strcmp(field,"db_key"))
	       strncpy(config->db_key, value,
		       sizeof(config->db_key));
	  else if(!strcmp(field,"db_port"))
	       config->db_port = atoi(value);
	  else if(!strcmp(field,"db_ip")) 
	       strncpy(config->db_ip, value,
		       sizeof(config->db_ip));
	  else if(!strcmp(field,"e_mail")) 
	       strncpy(config->e_mail, value,
		       sizeof(config->e_mail));
	  else if(!strcmp(field,"nodes"))
	       config->nodes = atoi(value);
	  else {
	       fprintf(stderr,"Unknown config file directive: %s\n",field);                    exit(1); }
     }

     free(fld_free);
     fclose(fp);

     /* probable mem leaks abound. */

     return 0;
}


char *str_up(char *str) {

     int pos;
     int len;
     static char *work;

     work = malloc(20);

     len = strlen(str);
     strncpy(work, str, 20);

     pos = 0;

     while(pos <= len) {
          work[pos] = toupper(work[pos]);
          pos++;
     }
     return work;
}

char *str_down(char *str) {

     int pos;
     int len;
     static char *work;

     work = malloc(20);

     len = strlen(str);
     strncpy(work, str, 20);

     pos = 0;

     while(pos <= len) {
          work[pos] = tolower(work[pos]);
          pos++;
     }
     return work;
}


int simple_copy(char *src, char *dest) {

     int srcfd, destfd;
     int len;
     char cpbuf[2048];
     
     if((srcfd = open(src, O_RDONLY)) == -1) {
	  printf("Couldn't open %s for read on copy.\n", src);
	  return -1;
     }

     if((destfd = open(dest, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR |
		       S_IWUSR | S_IRGRP)) == -1) {
	  printf("Couldn't open %s for write on copy\n", dest);
	  return -1;
     }

     while((len = read(srcfd, cpbuf, 2048)) > 0) 
	  write(destfd, cpbuf, len);
     
     close(srcfd);
     close(destfd);

     return 0;
}
	  
void netpause(int link) {

     char junk[80];

     read(link, junk, 80);
     return;
}

