/* primary controls for the database server.  get it loaded, set up
   the configuration, etc, etc... */

#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <signal.h>

#include <common.h>

#include "db.h"

/* globals for data storage */

BBS_Config *config;
struct ll_nodes *node_config;
struct ll_areas *area_config;
struct ll_nets *net_config;
struct ll_int *inc_list;
struct ll_files *files_config;
struct ll_protos *proto_config;

/* termination signal handler */
void db_exit(int sig);

/* make this global so we can close it nicely on termination */
static int sock;

main(int ac, char *av) {

     int n = 1;
     struct sockaddr_in sockstr;
     int socklen = sizeof(struct sockaddr_in);
     char logbuf[120];


     printf("%s DB server starting...\n", VERSION);
     /* ignore sigpipe so that the shit doesn't hit the fan from a
	misbehavin' child */
     signal(SIGPIPE, SIG_IGN);
     
     /* a SIGTERM or a SIGINT means we were killed or ctrl-c'd */
     signal(SIGTERM, db_exit);
     signal(SIGINT, db_exit);
     
     node_config = NULL;
     area_config = NULL;
     net_config = NULL;
     inc_list = NULL;
     
     /* config'em up! */
     
     config = (BBS_Config *)calloc(sizeof(BBS_Config), 1);
     read_system_config();

     read_msg_config();
     read_net_config();
     read_file_config();
     
     /* setup socket */
     if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	  perror("socket");
	  exit(127);
     }

     sockstr.sin_family = AF_INET;
     sockstr.sin_port = htons(config->db_port);
     sockstr.sin_addr.s_addr = inet_addr(config->db_ip);

     if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &n,
			 sizeof(n)) == -1) {
	  perror("setsockopt");
          exit(127);
     }
     
     if(bind(sock, (struct sockaddr *)&sockstr,
	     sizeof(struct sockaddr_in)) == -1) {
	  perror("bind");
	  exit(127);
     }

     if(listen(sock, 5) == -1) {
	  perror("listen");
	  exit(127);
     }

     printf("Startup finished, waiting for connections\n");
     sprintf(logbuf, "%s server started, pid %d\n", VERSION, getpid());
     db_log(255, logbuf);

     /* big ol parsing loop */

     while(1) {
	  int conn;
	  struct sockaddr_in newsockstr;
	  DB_Req req;

	  if((conn = accept(sock, &sockstr, &socklen)) == -1) {
	       perror("accept");
	       exit(127);
	  }

	  /* get their request */
	  if(read(conn, &req, sizeof(DB_Req)) != sizeof(DB_Req)) {
	       db_log(255, "Truncated DB request recieved");
	       shutdown(conn,2);
	       close(conn);
	       continue;
	  }

	  if(key_check(&req) != 0) {
	       DB_Ret ret;
	       
	       ret.result = INVALID_KEY;
	       ret.datasize = 0;
	       
	       write(conn, &ret, sizeof(DB_Ret));
	       db_log(255, "Invalid key presented");
	       shutdown(conn, 2);
	       close(conn);
	       continue;
	  }

	  /* alright, seems our key got verified, so do something with
	     it */

	  new_request(&req, conn);

	  /* all done, close 'er up. */
	  shutdown(conn, 2);
	  close(conn);

	  continue;
     }

/* after while loop..  never reached ATM... */

     db_exit(-1);

}

/* we're exiting for some reason, so clean it all up and go. */

void db_exit(int sig) {

     struct ll_nodes *cur;

     /* why are dying? */
     switch(sig) {
     case -1:
	  db_log(255, "Server terminating at client request\n");
	  fprintf(stderr, "Server terminating at client request\n");
	  break;
     case SIGINT:
	  db_log(255, "Server terminating on SIGINT\n");
	  fprintf(stderr, "Server terminating on SIGINT\n");
	  break;
     case SIGTERM:
	  db_log(255, "Server terminating on SIGTERM\n");
	  fprintf(stderr, "Server terminating on SIGTERM\n");
	  break;
     default:
	  db_log(255, "Server terminating by unknown cause... (?)\n");
	  fprintf(stderr, "Server terminating by unknown cause... (?)\n");
	  break;
     }

     close(sock);
     
     /* tell all of the BBS's that we're going down... */
     
     cur = node_config;

     while(cur != NULL) {
	  kill(cur->data.pid, SIGUSR1);
	  cur = cur->next;
     }

     /* and, ta ta... */

     if(sig == -1)
	  exit(0);
     else
	  exit(sig);


}
