/* handles figuring out what our request desires. */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>

#include "db.h"

extern BBS_Config *config;
extern struct ll_files *files_config;

/* handles a fresh request coming in from the outside */
/* assumes that the db key has been confirmed */

void new_request(DB_Req *req, int net) {

     DB_Ret ret;

     switch(req->req) {
     case NOOP:
	  ret.result = 0;
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     case RELOAD:
	  ret.result = reload_config();
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     case MIDNIGHT:
	  ret.result = midnight();
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     case LOGIN: {
	  BBS_Login info;
	  char *pair[2];

	  pair[0] = alloca(20);
	  pair[1] = alloca(20);

	  read(net, pair[0], 20);
	  read(net, pair[1], 20);

	  ret.result = do_login(&info, pair[0], pair[1]);
	  if(ret.result == 0) 
	       ret.datasize = sizeof(BBS_Login);
	  else
	       ret.datasize = 0;

	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(BBS_Login));
	  
	  break; }
     
     case USER_READ: {
	  BBS_User info;
	  char handle[20];
	  
	  read(net, handle, 20);
	  ret.result = db_user_read(&info, handle);
	  if(ret.result == 0) 
	       ret.datasize = sizeof(BBS_User);
	  else
	       ret.datasize = 0;

	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(BBS_User));
	  break;

     }
     case USER_WRITE: {
	  BBS_User info;

	  read(net, &info, sizeof(BBS_User));
	  ret.result = db_user_write(&info);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     case USER_CLEAR: {
	  char handle[20];

	  read(net, handle, 20);
	  ret.result = db_user_clear(handle);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     case USER_LIST: { 
	  int len, x;
	  BBS_User *list;

	  if((list = db_user_list(&len)) == NULL) {
	       ret.result = FAILURE;
	       ret.datasize = 0;
	       write(net, &ret, sizeof(DB_Ret));
	       break;
	  }
	  
	  ret.result = SUCCESS;
	  ret.datasize = len * sizeof(BBS_User);
	  write(net, &ret, sizeof(DB_Ret));

	  write(net, list, ret.datasize);

	  free(list);
	  break;

     }
     case USER_EMAIL_SCAN: {
	  char mail[60];

	  read(net, mail, 60);
	  ret.result = db_user_mail(mail);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     case NODE_READ: {
	  BBS_Node info;
	  int node;

	  read(net, &node, sizeof(int));
	  ret.result = db_node_read(&info, node);
	  if(ret.result == 0) 
	       ret.datasize = sizeof(BBS_Node);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(BBS_Node));
	  break; 
     }
     
     case NODE_WRITE: {
	  BBS_Node info;
	  
	  read(net, &info, sizeof(BBS_Node));
	  ret.result = db_node_write(&info);
	  ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     
     case NODE_CLEAR: {
	  int node;
	  
	  read(net, &node, sizeof(int));
	  ret.result = db_node_clear(node);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }

     case NODELIST_LOOKUP: {
	  tAddress lookup;
	  tConnect info;

	  read(net, &lookup, sizeof(tAddress));
	  ret.result = db_nodelist_lookup(&lookup, &info);
	  if(ret.result == 0)
	       ret.datasize = sizeof(tConnect);
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(tConnect));
	  break;
     }

     case POINT_READ: {
	  int info;
	  char index[80];

	  read(net, index, 80);
	  ret.result = db_point_read(&info, index);
	  if(ret.result == 0)
	       ret.datasize = sizeof(int);
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(int));
	  break;
     }
     case POINT_WRITE: {
	  int info;
	  char index[80];
	  
	  read(net, &info, sizeof(int));
	  read(net, index, 80);

	  ret.result = db_point_write(info, index);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     case POINT_CLEAR: {
	  char index[80];
	  int len;

	  read(net, index, 80);
	  ret.result = db_point_clear(index);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }

     case MSG_READ_HEAD: {
	  MSG_Head info;
	  MSG_Ref lookup;
	  
	  read(net, &lookup, sizeof(MSG_Ref));
	  ret.result = db_msg_read_head(&info, &lookup);
	  if(ret.result == 0)
	       ret.datasize = sizeof(MSG_Head);
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(MSG_Head));
	  break;
     }
     case MSG_READ: {
	  MSG_Ref lookup;
	  int len;
	  char *info;
	  
	  read(net, &lookup, sizeof(MSG_Ref));
	 
	  /* kludged because malloc hates it when you free a pointer
	     you got on a pass by reference. */
	  info = db_msg_read(&len, &lookup);
	  if(info == NULL) 
	       ret.result = FAILURE;
	  else
	       ret.result = SUCCESS;

	  if(ret.result == 0)
	       ret.datasize = len;
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  
	  if(ret.result == 0) {
	       write(net, info, len);
	       free(info);
	  }

	  break;
     }
     case MSG_WRITE: {
	  MSG_Ref where;
	  char *info;

	  read(net, &where, sizeof(MSG_Ref));
	  info = alloca(req->datasize);
	  read(net, info, req->datasize);
	  ret.result = db_msg_write(info, req->datasize, &where);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     case MSG_CLEAR: {
	  MSG_Ref lookup;

	  read(net, &lookup, sizeof(MSG_Ref));
	  ret.result = db_msg_clear(&lookup);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     case MSG_SCAN_USER: {
	  char handle[20];
	  MSG_Ref lookup;
	  int *msgs;
	  int len;
	  int x;
	  int bleh = 0;

	  read(net, &lookup, sizeof(MSG_Ref));
	  read(net, handle, 20);
	  ret.result == db_msg_scan_usr(&msgs, &len, handle, &lookup);
	  if(ret.result == 0)
	       ret.datasize = len*sizeof(int);
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0) {
	       x = 0;
	       while(x < len) {
		    write(net, &(msgs[x]), sizeof(int));
		    x++;
	       }
	       /* send a terminating 0 */
	       write(net, &bleh, sizeof(int));

	  free(msgs);

	  }
	  break;
     }
     case MSG_TOGGLE_TRANS: {
	  MSG_Ref lookup;
	  
	  read(net, &lookup, sizeof(MSG_Ref));
	  ret.result = db_msg_toggle_trans(&lookup);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }
     
     case MSG_AREA: {
	  MSG_Area info;
	  char name[20];
	  
	  read(net, name, 20);
	  ret.result = db_msg_area(&info, name);
	  if(ret.result == 0)
	       ret.datasize = sizeof(MSG_Area);
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(MSG_Area));
	  break;
     }

     case MSG_AREALIST: {
	  extern struct ll_areas *area_config;
	  struct ll_areas *cur;
	  int len = 0;

          /* count our nets */
          cur = area_config;
          while(cur != NULL) {
               len++;
               cur = cur->next;
          }
          if(len == 0)
               ret.result = FAILURE;
          else
               ret.result = 0;

	  if(ret.result == 0)
	       ret.datasize = len*sizeof(MSG_Area);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));

	  cur = area_config;
	  if(ret.result == 0) {
	       int x = 0;
               while(cur != NULL) {
                    /* last one? */
                    if((x+1) == len)
                         cur->data.done = 1;
                    write(net, &(cur->data), sizeof(MSG_Area));
                    x++;
                    cur = cur->next;
               }

	  }
	  break;
     }
     case MSG_AREA_NEXT:
     case MSG_AREA_PREV: {
	  MSG_Area cur, info;

	  read(net, &cur, sizeof(MSG_Area));
	  if(req->req == MSG_AREA_NEXT)
	       ret.result = db_msg_area_next(&cur, &info);
	  else
	       ret.result = db_msg_area_prev(&cur, &info);
	  if(ret.result == 0)
	       ret.datasize = sizeof(MSG_Area);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0) 
	       write(net, &info, sizeof(MSG_Area));
	  break;
     }
		     
     case MSG_NET: {
	  MSG_Net info;
	  char name[20];

	  read(net, name, 20);
	  ret.result = db_msg_net(&info, name);
	  if(ret.result == 0)
	       ret.datasize = sizeof(MSG_Net);
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0)
	       write(net, &info, sizeof(MSG_Net));

	  break;
     }

     case MSG_NETLIST: {
	  extern struct ll_nets *net_config;
	  int len = 0;
	  struct ll_nets *cur;
	  
	  /* count our nets */
	  cur = net_config;
	  do {
	       len++;
	       cur = cur->next;
	  } while(cur != NULL);

	  if(len == 0)
	       ret.result = FAILURE;
	  else
	       ret.result = 0;
	  
	  if(ret.result == 0)
	       ret.datasize = len*sizeof(MSG_Net);
	  else
	       ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));

	  cur = net_config;
	  if(ret.result == 0) {
	       int x = 0;
	       while(cur != NULL) {
		    /* last one? */
		    if(cur->next == NULL)
			 cur->data.done = 1;
		    write(net, &(cur->data), sizeof(MSG_Net));
		    x++;
		    cur = cur->next;
	       }
	  }
	  break;
     }
     
     case MENU_READ: {
	  BBS_Menu *entries;
	  char name[20];
	  int len;

	  read(net, name, 20);
	  ret.result = db_menu_read(&entries, &len, name);
	  if(ret.result == 0) 
	       ret.datasize = len*sizeof(BBS_Menu);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));
	  if(ret.result == 0) {
	       int x = 0;
	       while(x < len) {
		    /* last one? */
		    if((x+1) == len)
			 entries[x].done = 1;
		    else
			 entries[x].done = 0;
		    write(net, &(entries[x]), sizeof(BBS_Menu));
		    x++;
	       }
	       free(entries);

	  }

	  break;
     }
     
     case LOG_WRITE: {
	  char message[160];
	  
	  read(net, message, 160);
	  ret.result = db_log(req->node, message);
	  ret.datasize = 0;

	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }

     case INC_SEND: {
	  BBS_Inc info;

	  read(net, &info, sizeof(BBS_Inc));
	  ret.result = db_inc_send(&info);
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
     }

     case INC_PICKUP: {
	  BBS_Inc info;

	  ret.result = db_inc_pickup(&info, req->node);
	  if(ret.result == 0)
	       ret.datasize = sizeof(BBS_Inc);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));

	  if(ret.result == 0)
	       write(net, &info, sizeof(BBS_Inc));

	  break;
     }
     /* file areas */
     case FILE_AREA: {
	  FILE_Area info;
	  char name[20];
	  
	  read(net, name, 20);

	  ret.result = db_file_area(&info, name);
	  if(ret.result == 0)
	       ret.datasize = sizeof(FILE_Area);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));
	  
	  if(ret.result == 0)
	       write(net, &info, sizeof(FILE_Area));
	  
	  break;
     }

     case FILE_AREALIST:
     {
	  struct ll_files *cur;
	  int len = 0;

	  /* count up the number of areas for transmission. */
	  cur = files_config;

	  while(cur != NULL) {
	       len++;
	       cur = cur->next;
	  }
	  
	  if(len == 0) 
	       ret.result = FAILURE;
	  else
	       ret.result = SUCCESS;


	  if(ret.result == 0) 
	       ret.datasize = len*sizeof(FILE_Area);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));
	  
	  if(ret.result == 0) {
	       /* transmit each area. */
	       cur = files_config;
	       while(cur != NULL) {
		    write(net, &(cur->data), sizeof(FILE_Area));
		    cur = cur->next;
	       }
	  }
	  break;
     }
     case FILE_INFO:
     {
	  FILE_Info info;
	  
	  /* they send us a FILE_Info with only the area and
	     the filename, and we fill it, and also send the
	     description */

	  read(net, &info, sizeof(FILE_Info));
	  
	  ret.result = db_file_info(&info);

	  if(ret.result == 0) 
	       ret.datasize = sizeof(FILE_Info);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));
	  
	  if(ret.result != 0)
	       break;

	  write(net, &info, sizeof(FILE_Info));
	  write(net, info.desc, info.desclen);

	  free(info.desc);
	  break;
     }
     case FILE_LIST:
     {
	  FILE_Info *files;
	  int len = 0;
	  char area[20];

	  read(net, area, 20);

	  if((files = db_file_list(area, &len)) == NULL)
	       ret.result = FAILURE;
	  else
	       ret.result = SUCCESS;

	  if(ret.result == 0)
	       ret.datasize = len*sizeof(FILE_Info);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));

	  if(ret.result == 0) {
	       write(net, files, ret.datasize);
	       free(files);
	  }

	  break;
     }
     
     case FILE_ADD:
     {
	  FILE_Info info;

	  read(net, &info, sizeof(FILE_Info));
	  info.desc = malloc(info.desclen+1);
	  read(net, info.desc, info.desclen);
	  info.desc[info.desclen] = 0;

	  ret.result = db_file_add(&info);
	  ret.datasize = 0;

	  write(net, &ret, sizeof(DB_Ret));
	  
	  break;
     }

     case FILE_CLEAR:
     {
	  FILE_Info info;

	  read(net, &info, sizeof(FILE_Info));
	  ret.result = db_file_clear(&info);
	  ret.datasize = 0;

	  write(net, &ret, sizeof(DB_Ret));
	  
	  break;

     }

     case FILE_PROTOS:
     {
	  FILE_Proto *info;
	  int len;
	  int x = 0;

	  ret.result = db_file_protos(&info, &len);
	  if(ret.result == 0)
	       ret.datasize = sizeof(FILE_Proto)*len;
	  else
	       ret.datasize = 0;

	  write(net, &ret, sizeof(DB_Ret));

	  if(ret.result == 0) {
	       write(net, &len, sizeof(int));
	       while(x < len) {
	       write(net, &info, sizeof(FILE_Proto));
	       x++;
	       }
	       free(info);
	  }

	  break;
     }

     case FILE_NEXTAREA:
     case FILE_PREVAREA:
     {
	  struct ll_files *cur = files_config;
	  struct ll_files *prev = NULL;
	  FILE_Area info;

	  /* we succedded unless we decide so later. */
	  ret.result = 0;
	  
	  read(net, &info, sizeof(FILE_Area));
	  
	  while(cur != NULL) {
	       if(req->req == FILE_NEXTAREA) {
		    if(!strcasecmp(info.name, cur->data.name)) {
			 /* names match, pull out the next area, if
			    there is one. */
			 if(cur->next == NULL) 
			      break;

			 memcpy(&info, &(cur->next->data),
				sizeof(FILE_Area)); 
			 break;
		    }
	       }

	       if(req->req == FILE_PREVAREA) {
		    if(!strcasecmp(info.name, cur->data.name)) {
			 if(prev == NULL)
			      break;
			 
			 memcpy(&info, &(prev->data),
				sizeof(FILE_Area));
			 printf("previous hit: %s\n", info.name);
			 break;
		    }
	       }

	       prev = cur;
	       cur = cur->next;
	  }

	  /* did we find the existing area? */
	  if(cur == NULL)
	       ret.result = FAILURE;
	  
	  /* did we find what we were looking for? */
	  if((req->req == FILE_PREVAREA) && (prev == NULL))
	       ret.result = FAILURE;
	  else if((req->req == FILE_NEXTAREA) && (cur->next == NULL))
	       ret.result = FAILURE;

	  if(ret.result == 0)
	       ret.datasize = sizeof(FILE_Area);
	  else
	       ret.datasize = 0;
	  
	  write(net, &ret, sizeof(DB_Ret));

	  if(ret.result == 0) {
	       write(net, &info, sizeof(FILE_Area));
	       printf("next at %s, size %d\n", info.name, ret.datasize);
	  }

	  break;

     }

     /* any invalid requests should be trapped here */
     default: 
	  ret.result = UNK_REQ;
	  ret.datasize = 0;
	  write(net, &ret, sizeof(DB_Ret));
	  break;
	  
     } /* end switch */

     /* hopefully it all got processed, we'll leave it to the caller
	to close it all up. */
     
     return;

}

int key_check(DB_Req *test) {

     return strcmp(test->key, config->db_key);
}

	
