/* misc calls that don't deserve their own file. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "db.h"

extern BBS_Config *config;
extern struct ll_areas *area_config;
extern struct ll_nets *net_config;


int db_menu_read (BBS_Menu **menus, int *len, char *name){

     FILE *fp;
     char fn[60];
     

     sprintf(fn,"%s/menus/%s",config->base_dir, name);

     if((fp = fopen(fn,"r")) == NULL) {
	  fprintf(stderr,"attempt to open invalid menu: %s\n", name);
	  return FAILURE;
     }

     (*len) = 0;
     (*menus) = (BBS_Menu *)malloc(sizeof(BBS_Menu));
     
     while (fscanf(fp,"%c:%[^:]:%d:%d:%s:",&((*menus)[(*len)].key),
		   (*menus)[(*len)].label, &((*menus)[(*len)].access), 
		   &((*menus)[(*len)].type), (*menus)[(*len)].action)
	    != EOF) {
	  (*len)++;
	  (*menus) = realloc((*menus), sizeof(BBS_Menu)*((*len)+1));
	  fscanf(fp,"\n");
     }

     fclose(fp);

     return SUCCESS;

}

int db_log (int node, char *buffer) {
     
     FILE *fp;
     time_t secs;

     secs = time(NULL);

     if((fp = fopen(config->logfile,"a")) == NULL) 
	  return FAILURE;
     else 
	  fprintf(fp,"%s %d:%s",ctime(&secs), node, buffer);

     fclose(fp);
     
     return SUCCESS;
}

/* fix me */
int midnight(void) {

     BBS_User *user;
     BBS_User tmpuser;
     struct ll_areas *cur;

     DBM *users;
     datum key;
     datum data;
     int total_users = 0;

     int i;
     char buffer[80];

     db_log(255, "Running midnight maintenance...\n");
     
     sprintf(buffer,"%s/db/users.db",config->base_dir);

     users = dbm_open(buffer, O_RDWR | O_CREAT, S_IREAD | S_IWRITE);
     if(users == NULL) {
	  db_log(255, "Failed to open users.db for midnight maintenance.\n");
	  return FAILURE;
     }

     
     /* now, unfortunately, we do have to read every user record
        twice. We'll find a way around this in the future.  Now, we
        use the user modification pass as a way to total the number of
        users, then we'll build an array of them later. */

     key = dbm_firstkey(users);
     do {
	  user = (BBS_User *)malloc(sizeof(BBS_User));
          data = dbm_fetch(users,key);
	  memcpy(user, data.dptr, sizeof(BBS_User));
	  user->remaining = user->access;
	  data.dptr = (void *)user;
	  data.dsize = sizeof(BBS_User);
          dbm_store(users, key, data, DBM_REPLACE);
	  free(user);
          key = dbm_nextkey(users);
          total_users++;
     } while(key.dptr != NULL);

     /* alright, now make the big user array that renumber's gonna be using */

     user = (BBS_User *)malloc(sizeof(BBS_User) * total_users);
     i = 0;
     
     key = dbm_firstkey(users);
     do {
	  data = dbm_fetch(users, key);
	  memcpy(&(user[i]), data.dptr, sizeof(BBS_User));
	  key = dbm_nextkey(users);
	  i++;
     } while(key.dptr != NULL);

     dbm_close(users);
     
     /* right.  now, there should be all total_users users in there
	for renumber to play with. */
     
     cur = area_config;

     while(cur != NULL) {
	  renumber(&(cur->data), user, total_users);
	  cur = cur->next;
     }

     free(user);

     reload_config();

     db_log(255, "Maintnence complete.\n");
     
     return SUCCESS;
}

int reload_config(void) {

     db_log(255, "Reloading configs...\n");

     /* clear out all the message areas */
     
     while(area_config != NULL) {
          struct ll_areas *next;

          next = area_config->next;
          free(area_config);
          area_config = next;
     }

     /* and now the networks */
     while(net_config != NULL) {
          struct ll_nets *next;

          next = net_config->next;
          free(net_config);
          net_config = next;
     }

     read_msg_config();
     read_net_config();

     return SUCCESS;


}

int renumber(MSG_Area *area, BBS_User *users, int total_users) {

     int diff;
     int subs;
     int usernum;
     int curmsg;
     int lmsg;
     MSG_Ref lookup;
     
     strcpy(lookup.name, area->area_name);
     diff = area->last - area->expire;
     subs = 0;

     while(diff > 0) {
	  lookup.msg = diff;
          db_msg_clear(&lookup);
          subs++;
          diff--;
     }
     curmsg = area->first;
     lmsg = area->last;
     /* now, we should, hopefully, have the pointer to the actual
	thing in mem which has the area stats, so we'll affect when
	the other calls are doing. */

     area->last = 0; /* be sure it starts where we want */
     while(curmsg <= lmsg) {
          char *body;
          int size;

	  lookup.msg = curmsg;

	  /* look for dead messages */
	  if((body = db_msg_read(&size, &lookup)) == NULL) {
	       subs++;
	       curmsg++;
	       continue;
	  }

	  /* alright, now erase and rewrite at the end */
	  db_msg_clear(&lookup);
	  db_msg_write(body, size, &lookup);
	  free(body);
	  curmsg++;
	  continue;

     }


     if(subs != 0) {
	  usernum = 0;
	  
	  while(usernum < total_users) {
	       char *upointst;
	       int pmsg;
	       
               upointst = (char
			   *)malloc(strlen(users[usernum].handle) +
                                         strlen(area->area_name)+15);
               sprintf(upointst, "%s-,-%s", users[usernum].handle,
                       area->area_name);

               db_point_read(&pmsg, upointst);
               pmsg = pmsg - subs;
               if(pmsg < 0)
                    pmsg = 0;
               db_point_write(pmsg, upointst);

               free(upointst);
               usernum++;
          }
     }

     return SUCCESS;

}


