/* nodelist, read-only, for the database */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <string.h>

#include "db.h"

extern BBS_Config *config;

/* internal functions */
tNLIdx *index_lookup(tAddress *look);

/* looks up a single entry in the index.  a failure returns 65535, as
   it couldn't be a real location, with no room for an actual entry. */

tNLIdx *index_lookup(tAddress *look) {

     int indexfp;
     char filename[160];
     static tNLIdx *cur;
     int status;

     sprintf(filename, "%s/db/n2kidx.dat", config->base_dir);

     if((indexfp = open(filename, O_RDONLY)) == -1) {
	  return NULL;
     }

     cur = (tNLIdx *)malloc(sizeof(tNLIdx));

     while((status = read(indexfp, cur, sizeof(tNLIdx))) > 0) {
	  if((cur->Zone == look->Zone) && (cur->Net == look->Net))
	       break;
     }
     
     close(indexfp);

     if(status > 0) {
/*	  printf("found %d/%d in index\n", cur->Zone, cur->Net); */
	  return cur;
     }
     else {
/*	  printf("%d/%d not in index\n", cur->Zone, cur->Net); */
 	  return NULL;
     }	       
}

/* find a single entry in the nodelist.  requires that the connect
   type be specified...  for now. */

int db_nodelist_lookup(tAddress *lookup, tConnect *info) {

     tNLIdx *index;
     int file_pos, readstat;
     int datafp;
     char filename[160];
     static tNLDat *looking;
     char *type;
     
     /* temporarily hardcode the type to phone */
     type = strdup("CP");

     /* check for common munging alerts in a lookup and refuse to
	touch the fucker. */
     /* 200 sounds like a fair max for one's zone. */
     if((lookup->Zone > 200) || lookup->Zone < 1)
	  return FAILURE;
     
     /* oopsy, no type sepcified. */

     if(type == NULL) {
	  db_log(255, "nodelist lookup w/o connect type.\n");
	  return FAILURE;
     }

     /* hmmm..  if it's not in the index, we can't do anywhere. */
     
     if((index = index_lookup(lookup)) == NULL)
	  return FAILURE;

     
     if(!strcasecmp(type, "CE")) {
	  file_pos = index->EmailIdx*sizeof(tNLDat);
	  sprintf(filename, "%s/db/n2kemail.dat", config->base_dir);
     }
     else if(!strcasecmp(type, "CF")) {
	  file_pos = index->FTPIdx*sizeof(tNLDat);
	  sprintf(filename, "%s/db/n2kftp.dat", config->base_dir);
     }
     else if(!strcasecmp(type, "CP")) {
	  file_pos = index->PhoneIdx*sizeof(tNLDat);
	  sprintf(filename, "%s/db/n2kphone.dat", config->base_dir);
     }
     else if(!strcasecmp(type, "CT")) {
	  file_pos = index->TelnetIdx*sizeof(tNLDat);
	  sprintf(filename, "%s/db/n2ktel.dat", config->base_dir);
     }
     else if(!strcasecmp(type, "CI")) {
	  file_pos = index->ISDNIdx*sizeof(tNLDat);
	  sprintf(filename, "%s/db/n2kisdn.dat", config->base_dir);
     }
     else {
	  file_pos = index->OtherIdx*sizeof(tNLODat);
	  sprintf(filename, "%s/db/n2kother.dat", config->base_dir);
     }
     
     free(index);

     /* printf("an entry of type %s is at %d\n", type, file_pos); */
     /* good chance of getting a 0 here.  be suspicious of those. */
     
     if((datafp = open(filename, O_RDONLY)) == -1) {
	  printf("Failed opening %s, while looking up entry.\n", filename);
	  return FAILURE;
     }

     /* well, ok, the data file is there. */
     /* move us to the start of the entry in the file. */

     if(lseek(datafp, file_pos, SEEK_SET) == -1) {
	  printf("Couldn't move to needed point in file (%d).\n",
		 file_pos);
	  close(datafp);
	  return FAILURE;
     }
     
     /* ok, we're there.  Now, read in entries until one looks good to
	us. */
     
     looking = (tNLDat *)malloc(sizeof(tNLDat));

     do {
	  readstat = read(datafp, looking, sizeof(tNLDat));
	  if(readstat != sizeof(tNLDat))
	       break;
     }
     while((looking->Address.Zone != lookup->Zone) || (looking->Address.Net !=
	   lookup->Net) || (looking->Address.Node != lookup->Node) ||
	   (looking->Address.Point != lookup->Point));

     close(datafp);

     if(readstat != sizeof(tNLDat)) {
	  free(looking);
	  return FAILURE;
     }


     printf("found %hd:%hd/%hd.%hd@%s\n",
	    looking->Address.Zone,
	    looking->Address.Net, looking->Address.Node, looking->Address.Point,
	    looking->Address.Domain); 
     
     memcpy(info, &(looking->Connect), sizeof(tConnect));
     free(looking);

     return SUCCESS;

}
