
/* structures and definitions for db calls, for use by client and server. */
#include <sys/types.h>

/* making a request to the DB */
typedef struct _DB_Req {
     u_int8_t req; /* what are we requesting to happen? */
     u_int32_t datasize; /* request data size, 0 for pure queries. */
     char key[20]; /* access key for database */
     int node; /* what node wants to know? */
     int arg; /* optional argument to function */
} DB_Req;

/* DB reply to a request */
typedef struct _DB_Ret {
     u_int8_t result; /* result code for our request */
     u_int32_t datasize; /* size of return data, 0 if only confirms */
} DB_Ret;

/* request codes -- these do not need to be defined if we're in the
   BBS, that stuff doesn't care, and it can cause collisions. */

#ifndef IN_BBS

#define NOOP 0

/* action demands -- need node 0 and master key */
#define RELOAD 1
#define MIDNIGHT 2
#define LOGIN 3

/* user file requests */
#define USER_READ 10
#define USER_WRITE 11
#define USER_CLEAR 12
#define USER_LIST 13
#define USER_EMAIL_SCAN 14

/* node data */
#define NODE_READ 20
#define NODE_WRITE 21
#define NODE_CLEAR 22

/* message ops */
#define NODELIST_LOOKUP 30
#define POINT_READ 31
#define POINT_WRITE 32
#define POINT_CLEAR 33
#define MSG_READ_HEAD 34
#define MSG_READ 35
#define MSG_WRITE 36
#define MSG_CLEAR 37
#define MSG_SCAN_USER 38
#define MSG_TOGGLE_TRANS 39

/* msg area stuff */
#define MSG_AREA 40
#define MSG_AREALIST 41
#define MSG_AREA_NEXT 42
#define MSG_AREA_PREV 43
#define MSG_NET 44
#define MSG_NETLIST 45

/* menus */
#define MENU_READ 50

/* log */
#define LOG_WRITE 60

/* well, we always need these. */
#endif

/* inter-node comms */
#define INC_SEND 70
#define INC_PICKUP 71

#ifndef IN_BBS

/* file area controls */
#define FILE_AREA 80
#define FILE_AREALIST 81
#define FILE_INFO 82
#define FILE_LIST 83
#define FILE_ADD 84
#define FILE_CLEAR 85
#define FILE_PROTOS 86
#define FILE_NEXTAREA 87
#define FILE_PREVAREA 88

/* result codes */
#define SUCCESS 0 /* request processed successfully */
#define FAILURE 1 /* some failure in request */
#define INVALID_KEY 2 /* invalid key */
#define UNK_REQ 3 /* unknown request */
/* further result codes may be reserved for call-specific messages */

#endif
