
/* client controls to handle file areas */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <common.h>
#include <client.h>

extern int my_node;
extern BBS_Config *config;

FILE_Area *file_area(char *area) {

     FILE_Area *ret;
     int db_sock;
     DB_Req req;
     int size;

     req.req = FILE_AREA;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, area, 20);

     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return NULL;
     }

     ret = malloc(sizeof(FILE_Area));
     read(db_sock, ret, sizeof(FILE_Area));
     close(db_sock);
     
     return ret;
}

FILE_Area *file_arealist(int *len) {

     FILE_Area *ret;
     int db_sock;
     DB_Req req;
     int size, x;

     req.req = FILE_AREALIST;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;
     
     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return NULL;
     }

     (*len) = size/sizeof(FILE_Area);
     ret = malloc(size);
     
     big_read(db_sock, ret, size);

     close(db_sock);
     return ret;
}

int file_info(FILE_Info *lookup) {

     int db_sock;
     DB_Req req;
     int size, x;

     req.req = FILE_INFO;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, lookup, sizeof(FILE_Info));

     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return -1;
     }

     read(db_sock, lookup, sizeof(FILE_Info));
     lookup->desclen = size;
     lookup->desc = malloc(size+1);
     
     big_read(db_sock, lookup->desc, size);
     lookup->desc[size] = 0;

     close(db_sock);

     return 0;
}


FILE_Info *file_list(char *area, int *len) {

     FILE_Info *ret;
     int db_sock;
     DB_Req req;
     int size, x;

     req.req = FILE_LIST;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, area, 20);

     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return NULL;
     }

     (*len) = size/sizeof(FILE_Info);
     ret = malloc(size);

     big_read(db_sock, ret, size);

     close(db_sock);
     return ret;


}

int file_add(FILE_Info *info) {

     int db_sock;
     DB_Req req;
     int size, x;

     req.req = FILE_ADD;
     req.datasize = sizeof(FILE_Info);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, info, sizeof(FILE_Info));
     write(db_sock, info->desc, info->desclen);

     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return -1;
     }

     close(db_sock);
     return 0;

}

int file_clear(FILE_Info *info) {

     int db_sock;
     DB_Req req;
     int size, x;

     req.req = FILE_CLEAR;
     req.datasize = sizeof(FILE_Info);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, info, sizeof(FILE_Info));

     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return -1;
     }

     close(db_sock);
     return 0;


}

FILE_Proto *file_protos(int *len) {

     int db_sock;
     DB_Req req;
     int size, x;
     FILE_Proto *ret;

     req.req = FILE_PROTOS;
     req.datasize = 0;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;


     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return NULL;
     }

     ret = malloc(size);
     (*len) = size/sizeof(FILE_Proto);

     big_read(db_sock, ret, size);

     close(db_sock);
     
     return ret;

}

FILE_Area *file_nextarea(FILE_Area *cur) {

     int db_sock;
     DB_Req req;
     int size, x;
     FILE_Area *ret;

     req.req = FILE_NEXTAREA;
     req.datasize = sizeof(FILE_Area);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, cur, sizeof(FILE_Area));

     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return NULL;
     }

     ret = malloc(sizeof(FILE_Area));
     read(db_sock, ret, sizeof(FILE_Area));
     close(db_sock);

     return ret;


}

FILE_Area *file_prevarea(FILE_Area *cur) {

     int db_sock;
     DB_Req req;
     int size, x;
     FILE_Area *ret;

     req.req = FILE_PREVAREA;
     req.datasize = sizeof(FILE_Area);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, cur, sizeof(FILE_Area));

     if((size = db_reply(db_sock)) == -1) {
          close(db_sock);
          return NULL;
     }

     ret = malloc(sizeof(FILE_Area));
     read(db_sock, ret, sizeof(FILE_Area));
     close(db_sock);

     return ret;


}
