#include <stdio.h>
#include <curses.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <sys/socket.h>

#include <client.h>

extern BBS_Node *this_node;
extern BBS_Config *config;
extern int my_node;


MSG_Area *all_areas(void){

     MSG_Area *ret;
     int db_sock;
     DB_Req req;
     int size;
     int bleh;

     req.req = MSG_AREALIST;
     req.datasize = 0;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;
     
     if((size = db_reply(db_sock)) == -1) {
	  close(db_sock);
	  return NULL;
     }

     ret = (MSG_Area *)malloc(size);

     big_read(db_sock, ret, size);
     
     shutdown(db_sock, 2);
     close(db_sock);
     return ret;
     
}

MSG_Area *read_area(char *area) {

     MSG_Area *ret;
     int db_sock;
     DB_Req req;
     int size;

     req.req = MSG_AREA;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, area, 20);

     if(db_reply(db_sock) == -1)
          return NULL; 
     
     ret = (MSG_Area *)malloc(sizeof(MSG_Area));
     size = read(db_sock, ret, sizeof(MSG_Area));
     if(size < sizeof(MSG_Area)) {
	  shutdown(db_sock, 2);
	  close(db_sock);
	  free(ret);
	  return NULL;
     }
     
     shutdown(db_sock, 2);
     close(db_sock);
     return ret;

}

/* assumes that head's been allocated before the call */
char *read_message(MSG_Ref *lookup, MSG_Head *head) {

     int db_sock;
     int len;
     static char *ret;
     DB_Req req;
     int size;

     req.req = MSG_READ;
     req.datasize = sizeof(MSG_Ref);
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, lookup, sizeof(MSG_Ref));
     if((size = db_reply(db_sock)) == -1) {
	  close(db_sock);
          return NULL;
     }

     ret = (char *)calloc(sizeof(char *),(size+1)-sizeof(MSG_Head));

     read(db_sock, head, sizeof(MSG_Head));
     len = read(db_sock, ret, size);
     ret[len] = 0;

     shutdown(db_sock, 2);
     close(db_sock);

     return ret;
     
}

int write_message(MSG_Ref *where, MSG_Head *head, char *body) {

     int db_sock;
     DB_Req req;
     int size;
     char *xmit;

     req.req = MSG_WRITE;
     req.datasize = strlen(body)+sizeof(MSG_Head)+1;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;


     write(db_sock, where, sizeof(MSG_Ref));
     xmit = (char *)alloca(strlen(body)+sizeof(MSG_Head)+1);
     memcpy(xmit, head, sizeof(MSG_Head));
     memcpy(xmit+sizeof(MSG_Head), body, strlen(body));
     xmit[strlen(body)+sizeof(MSG_Head)] = 0;
     write(db_sock, xmit, strlen(body)+sizeof(MSG_Head)+1);
    
     if(db_reply(db_sock) == -1) {
	  close(db_sock);
	  return -1;
     }

     close(db_sock);     
     return 0;
     
}  

int clear_message(MSG_Ref *where) {
     
     int db_sock;
     DB_Req req;
     int size;

     req.req = MSG_CLEAR;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, where, sizeof(MSG_Ref));
     if(db_reply(db_sock) == -1) {
	  close(db_sock);
	  return -1;
     }

     shutdown(db_sock, 2);
     close(db_sock);
     return 0;

}

MSG_Net *read_net(char *name) {
     
     MSG_Net *ret;
     int db_sock;
     DB_Req req;
     int size;

     req.req = MSG_NET;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;

     write(db_sock, name, 20);
     if(db_reply(db_sock) == -1) {
	  close(db_sock);
          return NULL;
     }
     
     ret = (MSG_Net *)malloc(sizeof(MSG_Net));
     if(read(db_sock, ret, sizeof(MSG_Net)) < sizeof(MSG_Net)) {
	  free(ret);
	  ret = NULL;
     }

     shutdown(db_sock, 2);
     close(db_sock);
     
     return ret;
}


MSG_Net *all_nets(void) {

     MSG_Net *ret;
     int db_sock;
     int len;
     DB_Req req;
     int size;

     req.req = MSG_NETLIST;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;
     
     if((size = db_reply(db_sock)) == -1) {
	  close(db_sock);
	  return NULL;
     }

     ret = (MSG_Net *)malloc(size);

     big_read(db_sock, ret, size);

     shutdown(db_sock, 2);
     close(db_sock);
     return ret;

}

/* takes a fidonet address, and sees if it's ours in one of our nets */
MSG_Net *net_by_addr(char *addr) {

     MSG_Net *ret;
     MSG_Net *list;
     int x = 0;
     int found = 0;
     int zone = 0;
     int net = 0;
     int node = 0;
     int point = 0;

     list = all_nets();
     if(list == NULL) 
	  return NULL;

     /* extract the zone, net, node, point we're looking for */
     sscanf(addr, "%d:%d/%d.%d", &zone, &net, &node, &point);

     /* can't figure out a good conditional for this loop, I think I'm
	just being stupid right now. */

     while(1) {
	  /* check the address fields against the ones we just grabbed */
	  if((list[x].zone == zone) && (list[x].net == net) &&
	     (list[x].node == node) && (list[x].point == point)) {
	       found = 1;
	       break;
	  }
	  if(list[x].done == 1)
	       break;

	  x++;
     }  
     
     /* bail if it wasn't there */
     if(!found) {
	  free(list);
	  return NULL;
     }

     ret = (MSG_Net *)malloc(sizeof(MSG_Net));
     memcpy(ret, &(list[x]), sizeof(MSG_Net));
     free(list);

     return ret;
}
	    
int read_pointer(char *handle, char *area) {
     
     int db_sock, ret;
     char *buf;
     DB_Req req;
     int size;

     req.req = POINT_READ;
     req.datasize = 80;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     netprintf(db_sock, "%s-,-%s", handle, area);

     if(db_reply(db_sock) == -1) {
	  close(db_sock);
	  return -1;
     }

     read(db_sock, &ret, sizeof(int));
     shutdown(db_sock, 2);
     close(db_sock);

     return ret;

}

int clear_pointer(char *handle, char *area) {

     int db_sock, ret;

     DB_Req req;
     int size;

     req.req = POINT_CLEAR;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     netprintf(db_sock, "%s-,-%s", handle, area);

     ret = db_reply(db_sock);

     shutdown(db_sock, 2);
     close(db_sock);
     
     return ret;

}


int update_pointer(char *handle, char *area, int msg) {

     int db_sock, ret;
     DB_Req req;
     int size;

     req.req = POINT_WRITE;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;
     
     write(db_sock, &msg, sizeof(int));

     netprintf(db_sock, "%s-,-%s", handle, area);
     if(db_reply(db_sock) == -1) {
	  close(db_sock);
          return -1;
     }

     close(db_sock);

     return 0;

}

int tr_off(MSG_Ref *lookup) {

     int db_sock, ret;
     DB_Req req;
     int size;

     req.req = MSG_TOGGLE_TRANS;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return -1;

     write(db_sock, lookup, sizeof(MSG_Ref));
     ret = db_reply(db_sock);

     shutdown(db_sock, 2);
     close(db_sock);

     return ret;

}

MSG_Head *get_head(MSG_Ref *lookup) {

     int db_sock;
     MSG_Head *ret;
     DB_Req req;
     int size;

     req.req = MSG_READ_HEAD;
     req.datasize = 20;
     strcpy(req.key, config->db_key);
     req.node = my_node;

     if((db_sock = db_conn(&req)) == -1)
          return NULL;
     
     write(db_sock, lookup, sizeof(MSG_Ref));

     if(db_reply(db_sock) == -1) {
	  close(db_sock);
	  return NULL;
     }

     ret = (MSG_Head *)malloc(sizeof(MSG_Head));
     read(db_sock, ret, sizeof(MSG_Head));

     shutdown(db_sock, 2);
     close(db_sock);
     
     return ret;

}

MSG_Net *read_net_num(int netid) {

     MSG_Net *nets;
     MSG_Net *ret;
     int x = -1;

     nets = all_nets();

     do {
	  x++;
	  if(nets[x].netid == netid) 
	       break;
     } while(nets[x].done != 1);

     if(nets[x].netid != netid) {
	  free(nets);
	  return NULL;
     }

     ret = (MSG_Net *)malloc(sizeof(MSG_Net));
     memcpy(ret, &(nets[x]), sizeof(MSG_Net));
     free(nets);
     
     return ret;	  


}
