/* display routines for certain condidions in the bbs, such as
   prompts, errors, and the like, so that, when colors and stuff work,
   these will all be in the right color, and other little, easily
   tweakable things like that. */

#include <stdio.h>
#include <curses.h>
#include <stdlib.h>
#include <stdarg.h>

#include <common.h>
#include <client.h>

#include "disp.h"

extern BBS_Node *this_node;

/* add color controls to all of these once colors become sane. */

void bbserr(char *fmt, ...) {

     va_list argptr;

     va_start(argptr, fmt);
     vwprintw(stdscr, fmt, argptr);
     va_end(argptr);

     return;
}

void prompt(char *fmt, ...) {

     va_list argptr;

     va_start(argptr, fmt);
     vwprintw(stdscr, fmt, argptr);
     va_end(argptr);

     return;
}

int display_file(char *filename)
{
     FILE *fp;
     char ch[81];
     
     if((fp = fopen(filename,"r")) == NULL)
	  return 1;
     else 
	  while(fgets(ch,80,fp) != NULL)
	       addstr(ch);


     fclose(fp);
     return 0;
     
}     

void
right_justify(WINDOW * window, char * string)
{
  int x, y;
  char buffer[COLS];

  memset(buffer, ' ', sizeof(buffer));

  getyx(window, y, x);
  buffer[COLS-x-strlen(string)-1] = '\0';
  wprintw(window, "%s%s\n", buffer, string);
} /* right_justify */

void key_pause(void) {

     char junk;

     printw("Press any key to continue...\n");
     
     cbreak();
     junk = getch();
     nocbreak();

     return;
}


void prep_colors(BBS_User *user) {

     user->color[COL_PROMPT][0] = COLOR_CYAN;
     user->color[COL_PROMPT][1] = COLOR_BLACK;
     user->color[COL_PROMPT][2] = 0;
     user->color[COL_MENREG][0] = COLOR_GREEN;
     user->color[COL_MENREG][1] = COLOR_BLACK;
     user->color[COL_MENREG][2] = COL_BRIGHT;
     user->color[COL_MENHI][0] = COLOR_GREEN;
     user->color[COL_MENHI][1] = COLOR_BLACK;
     user->color[COL_MENHI][2] = 0;
     user->color[COL_SEP][0] = COLOR_YELLOW;
     user->color[COL_SEP][1] = COLOR_BLACK;
     user->color[COL_SEP][2] = COL_BRIGHT;
     user->color[COL_NAME][0] = COLOR_BLUE;
     user->color[COL_NAME][1] = COLOR_BLACK;
     user->color[COL_NAME][2] = COL_BRIGHT;
     user->color[COL_FLAG][0] = COLOR_RED;
     user->color[COL_FLAG][1] = COLOR_BLACK;
     user->color[COL_FLAG][2] = COL_BRIGHT;
     user->color[COL_LABEL][0] = COLOR_MAGENTA;
     user->color[COL_LABEL][1] = COLOR_BLACK;
     user->color[COL_LABEL][2] = COL_BRIGHT;
     user->color[COL_PRHI][0] = COLOR_WHITE;
     user->color[COL_PRHI][1] = COLOR_BLACK;
     user->color[COL_PRHI][2] = COL_BRIGHT;
     user->color[COL_NORM][0] = COLOR_WHITE;
     user->color[COL_NORM][1] = COLOR_BLACK;
     user->color[COL_NORM][2] = 0;
}

void init_pairs(BBS_User *user) {

     int x;

     x = 0;
     while (x < 16) {
	  init_pair(x, user->color[x][0],
		    user->color[x][1]);
	  x++;
     }
     

}     

void cprintw(short color, char *fmt, ...) {

     va_list argptr;

     color_set(color, NULL);
     if(this_node->current_user->color[color][2] & COL_BLINK)
	  attron(A_BLINK);
     else
	  attroff(A_BLINK);
     if(this_node->current_user->color[color][2] & COL_BRIGHT)
          attron(A_BOLD);
     else
	  attroff(A_BOLD);


     va_start(argptr, fmt);
     vwprintw(stdscr, fmt, argptr);
     va_end(argptr);

     return;

}

char hotch(void) {

     char ret;

/*     if(this_node->current_user->flags & FL_HOTKEY) { */
	  cbreak();
	  ret = getch();
	  nocbreak();
	  return ret;
/*     }
     else {
	  ret = getch();
	  nodelay(stdscr, TRUE);
	  while(getch() != ERR);
	  nodelay(stdscr, FALSE);
	  return ret;
	  } */
     
}


void color_config(BBS_User *user) {

     char opt;
     short dome;
     int x;

     opt = 0;

     while(opt != 'q') {

	  opt = 0;
	  cprintw(COL_NORM, "1)Normal Text\n");
	  cprintw(COL_PROMPT, "2)Prompts:\n");
	  cprintw(COL_MENREG, "3/");
	  cprintw(COL_MENHI, "4)");
	  cprintw(COL_MENREG, "Menu normal/highlight\n");
	  cprintw(COL_SEP, "5)--------------- Seperation Lines\n");
	  cprintw(COL_NAME, "6)User Names (%s)\n", user->handle);
	  cprintw(COL_FLAG, "7)Boolean Flags    Pvt Local Sent\n");
	  cprintw(COL_LABEL, "8)Field labels   Subject:\n");
	  cprintw(COL_VAL1, "9)Misc output fields\n");
	  cprintw(COL_NORM, "r)eset colors\n");
	  cprintw(COL_NORM, "q)quit\n");
	  cprintw(COL_PROMPT, "\nchange: ");
	  dome = 255;

	  opt = hotch();
	  
	  if(opt == '1')
	       dome = COL_NORM;
	  else if(opt == '2')
	       dome = COL_PROMPT;
	  else if(opt == '3')
	       dome = COL_MENREG;
	  else if(opt == '4')
	       dome = COL_MENHI;
	  else if(opt == '5')
	       dome = COL_SEP;
	  else if(opt == '6')
	       dome = COL_NAME;
	  else if(opt == '7')
	       dome = COL_FLAG;
	  else if(opt == '8')
	       dome = COL_LABEL;
	  else if(opt == '9')
	       dome = COL_VAL1;
 
	  if(dome != 255) {
	       cprintw(COL_NORM, "\n%d: black\n", COLOR_BLACK);
	       cprintw(COL_NORM, "%d: red\n", COLOR_RED);
	       cprintw(COL_NORM, "%d: green\n", COLOR_GREEN);
	       cprintw(COL_NORM, "%d: yellow\n", COLOR_YELLOW);	
	       cprintw(COL_NORM, "%d: blue\n", COLOR_BLUE);
	       cprintw(COL_NORM, "%d: magenta\n", COLOR_MAGENTA);
	       cprintw(COL_NORM, "%d: cyan\n", COLOR_CYAN);
	       cprintw(COL_NORM, "%d: white\n", COLOR_WHITE);
	       cprintw(COL_NORM, "%d: %d on %d\n",
		       dome, user->color[dome][0], user->color[dome][1]);
	       cprintw(COL_PROMPT, "\nforeground: ");
	       scanw("%hd", &user->color[dome][0]);
	       cprintw(COL_PROMPT, "\nbackground: ");
	       scanw("%hd", &user->color[dome][1]);
	       cprintw(COL_NORM, "0:normal\n");
	       cprintw(COL_NORM, "1:blink\n");
	       cprintw(COL_NORM, "2:bright\n");
	       cprintw(COL_NORM, "3:bright+blink\n");
	       cprintw(COL_NORM, "attribs: ");
	       scanw("%hd", &x);
	       if(x == 2) 
		    user->color[dome][2] = COL_BRIGHT;
	       init_pairs(user);
	  }
	  else if(opt == 'q') 
	       return;
	  else if(opt == 'r')
	       prep_colors(user);
	  else
	       cprintw(COL_NORM, "Invalid selection.\n");

     }

}

