/* tosser.c -- uncompresses incoming echomail archives, and tosses
   fido-style *.pkt files into the *.msg message areas.  The methods
   by which it does this are basically based on what sounded good at
   the time.  there is probably substantial room for improvement. */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdlib.h>

/* and the local includes */

#include <common.h>
#include <client.h>

#include "tosser.h"

/* define to spit copious debug info */
#define DEBUG

BBS_Config *config;
BBS_Node *this_node;
int my_node = 0;
struct toss_config *mconfig;

main(int ac, char *av[]) {

     int num;
     struct dirent **packets;
     int x = 0;

     config = (BBS_Config *)malloc(sizeof(BBS_Config));
     read_system_config();
     
     mconfig = (struct toss_config *)malloc(sizeof(struct toss_config));
     read_toss_config();

     decomp_inbound();

     /* find all of the *.pkt files */
     num = scandir(mconfig->incoming, &packets, pkt_match, alphasort);

     if(num == -1) {
	  printf("Error accessing incoming directory.\n");
     }

     printf("Found %d total incoming packets.\n",num);

     /* iterate through the packets, tossing what we find */

     while(x < num) {

	  MSG_Pack pack_head;
	  MSG_Head head;

	  char pathname[80];
	  int len;

	  char *body;
	  char *contents;
	  char **next;
	  char *partial;
	  char *area;

	  sprintf(pathname, "%s/%s", mconfig->incoming,
		 packets[x]->d_name);	  

	  /* pull out the packet header */
	  if((contents = read_packet(pathname, &pack_head))
	     == NULL) {
	       printf("Invalid packet %s, skipping...\n",packets[x]->d_name);
	       x++;
	       continue;
	  }

	  /* we've now got the packet in memory, header pulled off.
	     move through the packet tossing messages until we reach
	     the end of the giant string */

	  next = &contents;
	  while(*next != NULL)
	  {
	       body = get_msg(next, area, &head); 
	       toss_msg(&head, body);	  
	  }
	  
	  x++;
	  free(contents); 
	  /* alright, we should have tossed all packets now, so let's
	     delete the .pkt which they were in. */
	  unlink(pathname);

     }
     
}
     
 
/* look for compressed packets.  lamely. :) */

int comp_match(struct dirent *entry) {

     int ret;
     
     if(strstr(entry->d_name, ".su") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".mo") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".tu") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".we") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".th") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".fr") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".sa") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".SU") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".MO") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".TU") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".WE") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".TH") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".FR") != NULL) 
	  return 1;
     else if(strstr(entry->d_name, ".SA") != NULL) 
	  return 1;
     else
	  return 0;

}

/* reads a packet into memory from disk */
char *read_packet(char *filename, MSG_Pack *head) {

     int packfd;
     int len;
     static char *ret;
     struct stat filesize;

     /* open up the file containing the packet... */

     printf("Reading packet %s...\n", filename);
     if((packfd = open(filename, O_RDONLY)) == -1) {
	  perror("read_packet");
	  return NULL;
     }

     /* stat so that we can get the size, then, find the size of the
	total packet body (w/o header) which is what we're really
	concerned about here. */

     fstat(packfd, &filesize);

     len = filesize.st_size - sizeof(MSG_Pack);

     /* set up our memory for the rest of the packet, and read it in
	to memory. */

     ret = (char *)malloc(len);

     if(read(packfd, head, sizeof(MSG_Pack)) < sizeof(MSG_Pack)) {
	  printf("Broken packet %s, not even a header.\n", filename);
	  close(packfd);
	  free(ret);
	  return NULL;
     }

     read(packfd, ret, len);

     close(packfd);

     return ret;

}

/* moves through the memory area containing the messages within a
   packet, building a MSG_Head and returning the body */

char *get_msg(char **next, char *area, MSG_Head *head) {

     char *body;
     MSG_Head_Pack *phead;
     int bsize;
     char *src;

/*     if(*next == 0) 
       return NULL; */

     /* alright, we ought to have a header immediately at *next */
     
     phead = (MSG_Head_Pack *)calloc(sizeof(MSG_Head_Pack),1);
     /* and trim off the last 144 bytes, those are padded out too far
	here. */
     /* 144 = to[36] + from[36] + subject[36] */

     memcpy(phead, *next, sizeof(MSG_Head_Pack) - 144);

     /* move the pointer past the bit we copied, and we've got the to... */

     *next = *next + sizeof(MSG_Head_Pack) - 144;
     memcpy(phead->to, *next, strlen(*next)+1); /* copy \0 */
     *next = *next + strlen(*next)+1;
     memcpy(phead->from, *next, strlen(*next)+1); /* again */
     *next = *next + strlen(*next)+1;
     memcpy(phead->subject, *next, strlen(*next)+1); /* again */
     *next = *next + strlen(*next)+1;

     /* alright, the psuedo-header is all filled up. Build the real
	one from it. */
     
     strcpy(head->to, phead->to);
     strcpy(head->from, phead->from);
     strcpy(head->subject, phead->subject);
     strncpy(head->date, phead->date, 20);
     
     /* the net and node stuff will be fixed up after our return,
	based on the actual packet. */
     head->attribute = phead->attribute;
     head->cost = phead->cost;

     /* alright, head's configged.  now fix up the body.  ick. */
     
/*     bsize = (strchr(*next, 0) - (*next+1)); */
     bsize = strlen(*next)+1;
     body = (char *)calloc(bsize,1);
     memcpy(body, *next, bsize);
     *next = *next + bsize;
     src = msgid_src(body);

     if(src != NULL) {
	  if(sscanf(src, "%hd:%hd/%hd.%hd", &head->origzone, &head->orignet,
		    &head->orignode, &head->origpoint) < 4)
	       head->origpoint = 0;
     } else {
	  head->origzone = 0;
	  head->orignet = 0;
	  head->orignode = 0;
	  head->origpoint = 0;
     }
     
     
     /* hmmm... now, with any luck, *next points to the start of a
	brand new header. */
     
     if(*next[0] != 2) {  
	  *next = NULL;
     }

     return body;
}

/* puts a message where it belongs, based on information found in the
   body */

void toss_msg(MSG_Head *head, char *body) {

     MSG_Area *area;
     MSG_Ref where;
     char *fixch;

     where.msg = 0;
     
     if(dupe_check(body)) {
	  strcpy(where.name, mconfig->dupes);
	  write_message(&where, head, body); 
	  return; 
     }

     if((area = get_area(body)) == NULL) {
	  printf("Message for invalid group %s, how?",
		 area->area_name);
/*	  invalid(head, body); */
	  return;
     }

/*     while((fixch = strchr(body, 1)) != NULL) {
	  fixch[0] = ' ';
	  addch('.');
     }
*/
     putchar('\n');
     
     if(head->attribute & MSG_TRANSIT)
	  head->attribute ^= MSG_TRANSIT;

     strcpy(where.name, area->area_name);
     write_message(&where, head, body); 
     return;

}

/* phew, this is a difficult to read hunk of shit here.  basically, it
   extracts an area tag, or, if there's an INTL tag, decides it's
   netmail and does that.  */

MSG_Area *get_area(char *body) {

     char *area;
     char *tmp;
     char *work;

     work = (char *)malloc(strlen(body)+1);
     strcpy(work, body);

     area = (char *)calloc(sizeof(char), 20);
     
     tmp = strstr(work, "AREA:");
     if(tmp == NULL) {
	  char *dest;
	  char *tmp2;
	  MSG_Net *net;

	  printf("Netmail message...\n");
	  tmp = strstr(work, "INTL ");
	  tmp2 = strsep(&tmp, "\r");
#ifdef DEBUG
	  printf("netmail line: %s\n", tmp2);
#endif
	  dest = (char *)calloc(sizeof(char),80);
	  sscanf(tmp2, "INTL %s", dest);
	  net = net_by_addr(dest);
	  if(net == NULL) {
	       printf("netmail not for us? (%s)\n", dest);
	       strcpy(area, "junk");
	  } else 
	       strcpy(area, net->netmail);
     } else {
	  char *tmp2;
	  int len;
	  tmp2 = strsep(&tmp, "\r");
	  len = strlen(tmp2);
	  memcpy(area, tmp2+5, len-4);  
     }

     free(work);
     printf("area: %s\n", area);
     return read_area(area); 


}
     
int dupe_check(char *body) {

     char pathname[80];
     DBM *db;
     char *msgid;
     char *tmp;
     char *tmp2;
     char *work;
     datum id;
     datum bleh;

     work = (char *)malloc(strlen(body)+1);
     strcpy(work, body);
     tmp = strstr(work, "MSGID:");
     tmp2 = strsep(&tmp, "\r");
     if((tmp == NULL) || (tmp2 == NULL))
	  return 0;
     
     id.dptr = (void *)tmp2;
     id.dsize = strlen(tmp2);

     sprintf(pathname, "%s/db/dupes.db", config->base_dir);


     db = dbm_open(pathname, O_RDWR | O_CREAT, S_IREAD | S_IWRITE);
     if(db == NULL) {
	  printf("Couldn't open dupes DB.\n");
	  return 0;
     }

     bleh = dbm_fetch(db, id);
     if(bleh.dptr != 0) {
	  printf("Dupe! %s\n", tmp2);
	  dbm_close(db);
	  free(work);
	  return 1;

     }
     
     dbm_store(db, id, id, DBM_REPLACE);

     dbm_close(db);
     free(work);
     return 0;
     
}


char *msgid_src(char *body) {

     char *tmp;
     char *tmp2;
     char *work;
     char *src;

     work = (char *)malloc(strlen(body)+1);
     strcpy(work, body);
     tmp = strstr(work, "MSGID:");
     tmp2 = strsep(&tmp, "\r");
     if((tmp == NULL) || (tmp2 == NULL))
	  return NULL;
     src = (char *)calloc(sizeof(char),80);
     sscanf(tmp2, "MSGID: %s", src);

     free(work);

     return src;

}

void decomp_inbound(void) {

     struct dirent **names;
     int num;

     num = scandir(mconfig->incoming, &names, comp_match, alphasort);

     chdir(mconfig->incoming);
     

     while(num > 0) {
	  char *cmdline;

	  /* allocate space for max decomp command+15 for name */
	  cmdline = (char *)malloc(95);
	  num--;

	  /* zip currently hardcoded. :( */

	  sprintf(cmdline, "%s %s","unzip -o -L", names[num]->d_name);
	  system(cmdline);

	  /* done decompressing, delete archive. */
	  unlink(names[num]->d_name);
	  free(cmdline);

     }	  

     return;

}

void read_toss_config(void) {

     FILE *fp;
     char field[80];
     char value[80];
     char filename[80];
     char *env;

     env = getenv("ox_config");
     if(env == NULL)
          env = getenv("OX_CONFIG");
     if(env != NULL)
          sprintf(filename, "%s/mail.ox", strsep(&env, "= "));
     else
          /* assume we're one subdir lower than where the config lies */
          sprintf(filename, "config/mail.ox");

     if ((fp = fopen(filename,"r")) == NULL) {
	  perror("read_toss_config");
          exit(127);
     }
     else {

          /* error check these fuckers eventually when we insert
             them. */

          while(fscanf(fp,"%s %s",field,value) != EOF) {
               if(!strcmp(field,"incoming")) {
                    strncpy(mconfig->incoming, value,
                            sizeof(mconfig->incoming)); }
               else if(!strcmp(field, "dupes"))
                    strncpy(mconfig->dupes, value,
                            sizeof(mconfig->dupes));
               else if(!strcmp(field, "outgoing"))
                    strncpy(mconfig->outgoing, value,
                            sizeof(mconfig->outgoing));
               else {
                    printf("Unknown mail config file directive: %s\n",
                            field);
                    exit(127); }
          }

     }
}


int pkt_match(struct dirent *entry) {

     int ret;

     if((strstr(entry->d_name, ".pkt") != NULL) ||
	(strstr(entry->d_name, ".PKT") != NULL ))
          return 1;
     else
          return 0;

}

